"""
SSE 适配器核心组件
===============

提供SSE适配器的核心功能实现。
"""

from .client import SseClient, SseClientManager
from .message import SseMessageConverter
from .service import SseApiService

__all__ = ["SseApiService", "SseClient", "SseClientManager", "SseMessageConverter"]



import json
from typing import Optional

from pydantic import BaseModel

from nekro_agent.services.agent.openai import OpenAIResponse


class SandboxCodeExtData(BaseModel):
    message_cnt: int
    token_consumption: int
    token_input: int
    token_output: int
    chars_count_input: int
    chars_count_output: int
    chars_count_total: int
    use_model: str
    speed_tokens_per_second: float
    speed_chars_per_second: float
    first_token_cost_ms: int
    generation_time_ms: int
    stream_mode: bool
    log_path: str = ""

    @classmethod
    def create_from_llm_response(
        cls,
        llm_response: OpenAIResponse,
    ) -> "SandboxCodeExtData":
        speed_chars_per_second = (
            len(llm_response.response_content) / (llm_response.generation_time_ms / 1000)
            if llm_response.generation_time_ms > 0
            else 0
        )
        speed_chars_per_second = round(speed_chars_per_second, 1)

        prompt_str = ""
        if llm_response.messages:
            for message in llm_response.messages:
                if isinstance(message, dict):
                    content = message.get("content", "")
                    if isinstance(content, str):
                        prompt_str += content
                    elif isinstance(content, list):
                        for item in content:
                            if isinstance(item, dict) and item.get("type") == "text":
                                prompt_str += item.get("text", "")

        chars_count_input = len(prompt_str)
        chars_count_output = len(llm_response.response_content)
        chars_count_total = chars_count_input + chars_count_output

        return cls(
            message_cnt=llm_response.message_cnt,
            token_consumption=llm_response.token_consumption,
            token_input=llm_response.token_input,
            token_output=llm_response.token_output,
            chars_count_input=chars_count_input,
            chars_count_output=chars_count_output,
            chars_count_total=chars_count_total,
            use_model=llm_response.use_model,
            speed_tokens_per_second=llm_response.speed_tokens_per_second,
            speed_chars_per_second=speed_chars_per_second,
            first_token_cost_ms=llm_response.first_token_cost_ms,
            generation_time_ms=llm_response.generation_time_ms,
            stream_mode=llm_response.stream_mode,
            log_path=str(llm_response.log_path) if llm_response.log_path else "",
        )

    def model_dump_json(self) -> str:
        return json.dumps(self.model_dump(), ensure_ascii=False)



from pydantic import Field, SecretStr

from nekro_agent.adapters.interface.base import BaseAdapterConfig


class DiscordConfig(BaseAdapterConfig):
    """Discord 适配器配置"""

    BOT_TOKEN: str = Field(
        default="",
        title="Discord Bot Token",
        description="从 <a href='https://discord.com/developers/applications' target='_blank'>Discord Developer Portal</a> 获取你的 Bot Token",
    )



"""
# Minecraft 工具集 (Minecraft Utils)

为 `Minecraft` 适配器提供一系列专属的增强功能，允许 AI 与游戏服务器进行深度交互。

## 主要功能

- **发送富文本消息**: AI 可以构建一种特殊的 JSON 格式消息，在游戏中显示带有颜色、样式（粗体、斜体）、点击事件（如打开网页、执行命令）和悬停提示的复杂文本。这可以用来创建交互式公告、任务指引等。
- **执行 RCON 命令**: AI 能够通过 RCON 协议直接向 Minecraft 服务器发送指令。这意味着 AI 可以像一个游戏管理员（OP）一样，执行诸如 `/weather clear` (改变天气)、`/give` (给予物品)、`/tp` (传送玩家) 等各种原版指令。

## 使用方法

此插件的所有工具都是为 AI 自动调用而设计的，用户无需手动操作。AI 会根据需要与 Minecraft 服务器进行交互：

- 当需要发送复杂的交互式信息时，AI 会构造富文本并调用 `send_rich_text`。
- 当需要改变游戏世界的状态或与玩家互动时，AI 会调用 `execute_rcon_commands` 来执行相应的指令。

通过这个插件，AI 可以在很大程度上成为一个合格的 Minecraft 服务器"服主"或"游戏管理员"。
"""

import json
from typing import Any, Dict, List, Optional, Union

from nonebot.adapters.minecraft import Bot, Message, MessageSegment
from nonebot.adapters.minecraft.model import (
    BaseComponent,
    ClickAction,
    ClickEvent,
    HoverAction,
    HoverEvent,
    TextColor,
    TextComponent,
)

from nekro_agent.adapters.minecraft.core.bot import get_bot
from nekro_agent.api import core
from nekro_agent.api.core import logger
from nekro_agent.api.plugin import NekroPlugin, SandboxMethod, SandboxMethodType
from nekro_agent.api.schemas import AgentCtx
from nekro_agent.models.db_chat_channel import DBChatChannel
from nekro_agent.services.message_service import message_service

RCON_ERROR_PREFIXES = (
    "Unknown or incomplete command",
    "Incorrect argument",
    "Invalid player",
    "Player not found",
    "That player is not online",
    "You do not have permission to use this command",
    "Cannot give",
    "Invalid UUID",
    "No such entity",
    "That block is not a container",
    "Could not insert items",
    "Data tag parsing failed",
    "Expected",
    "Invalid command syntax",
    "An unexpected error occurred",
    "No targets matched selector",
    "The entity UUID is invalid",
    "Invalid command format",
)

plugin = NekroPlugin(
    name="Minecraft 工具插件",
    module_name="minecraft_utils",
    description="提供 Minecraft 适配器专用的消息发送和 RCON 命令执行功能。",
    version="0.1.0",
    author="Zaxpris",
    url="https://github.com/KroMiose/nekro-agent",
    support_adapter=["minecraft"],
)


def _llm_segment_dict_to_text_component(segment_dict: Dict[str, Any]) -> TextComponent:
    """
    将 LLM 输出的单个富文本段字典转换为 Minecraft TextComponent。
    主要用于构建 HoverEvent.text 中的组件。
    """
    text = segment_dict.get("text", "")
    component_data: Dict[str, Any] = {"text": text}

    if "color" in segment_dict:
        try:
            component_data["color"] = TextColor(str(segment_dict["color"]).lower())
        except ValueError:
            logger.warning(
                f"[MinecraftUtils] 无效的悬停文本颜色: {segment_dict['color']}",
            )

    for style_attr in ["bold", "italic", "underlined", "strikethrough", "obfuscated"]:
        if style_attr in segment_dict:
            component_data[style_attr] = segment_dict[style_attr]

    if "insertion" in segment_dict:
        component_data["insertion"] = segment_dict["insertion"]

    click_action_str = segment_dict.get("click_event_action")
    click_value = segment_dict.get("click_event_value")
    if click_action_str and click_value:
        try:
            action = ClickAction(str(click_action_str).lower())
            component_data["click_event"] = ClickEvent(
                action=action,
                value=str(click_value),
            )
        except ValueError:
            logger.warning(
                f"[MinecraftUtils] 无效的悬停点击事件动作: {click_action_str}",
            )

    hover_action_str = segment_dict.get("hover_event_action")
    if hover_action_str:
        try:
            hover_action = HoverAction(str(hover_action_str).lower())
            if hover_action == HoverAction.SHOW_TEXT:
                hover_text_segments_data = segment_dict.get("hover_event_text_segments")
                if isinstance(hover_text_segments_data, list):
                    nested_hover_components: List[BaseComponent] = [
                        _llm_segment_dict_to_text_component(s) for s in hover_text_segments_data
                    ]
                    component_data["hover_event"] = HoverEvent(
                        action=hover_action,
                        text=nested_hover_components,
                    )
                else:
                    logger.warning(
                        "[MinecraftUtils] hover_event_text_segments 在悬停事件中缺失或非列表",
                    )
            # TODO: 支持其他 HoverAction (show_item, show_entity) 如果需要
        except ValueError:
            logger.warning(
                f"[MinecraftUtils] 无效的悬停事件动作: {hover_action_str}",
            )
    return TextComponent(**{k: v for k, v in component_data.items() if v is not None})


def _llm_segment_to_mc_message_segment(
    segment_dict: Dict[str, Any],
) -> Optional[MessageSegment]:
    """
    将 LLM 输出的单个富文本段字典转换为 Minecraft MessageSegment。
    """
    text = segment_dict.get("text")
    if text is None:  # 每个段必须有文本
        logger.warning("[MinecraftUtils] 富文本段缺少 'text' 键。")
        return None

    segment_args: Dict[str, Any] = {}

    if "color" in segment_dict:
        try:
            segment_args["color"] = TextColor(str(segment_dict["color"]).lower())
        except ValueError:
            logger.warning(
                f"[MinecraftUtils] 无效的文本颜色: {segment_dict['color']}",
            )

    for style_attr in ["bold", "italic", "underlined", "strikethrough", "obfuscated"]:
        if style_attr in segment_dict:
            segment_args[style_attr] = segment_dict[style_attr]

    if "insertion" in segment_dict:
        segment_args["insertion"] = segment_dict["insertion"]

    click_action_str = segment_dict.get("click_event_action")
    click_value = segment_dict.get("click_event_value")
    if click_action_str and click_value:
        try:
            action = ClickAction(str(click_action_str).lower())
            segment_args["click_event"] = ClickEvent(
                action=action,
                value=str(click_value),
            )
        except ValueError:
            logger.warning(
                f"[MinecraftUtils] 无效的点击事件动作: {click_action_str}",
            )

    hover_action_str = segment_dict.get("hover_event_action")
    if hover_action_str:
        try:
            hover_action = HoverAction(str(hover_action_str).lower())
            if hover_action == HoverAction.SHOW_TEXT:
                hover_text_segments_data = segment_dict.get("hover_event_text_segments")
                if isinstance(hover_text_segments_data, list):
                    hover_components: List[BaseComponent] = [
                        _llm_segment_dict_to_text_component(s_dict) for s_dict in hover_text_segments_data
                    ]
                    segment_args["hover_event"] = HoverEvent(
                        action=hover_action,
                        text=hover_components,
                    )
                else:
                    logger.warning(
                        "[MinecraftUtils] hover_event_text_segments 缺失或非列表",
                    )
            # TODO: 支持其他 HoverAction (show_item, show_entity) 如果需要
        except ValueError:
            logger.warning(
                f"[MinecraftUtils] 无效的悬停事件动作: {hover_action_str}",
            )

    return MessageSegment.text(str(text), **segment_args)


@plugin.mount_sandbox_method(
    SandboxMethodType.TOOL,
    name="发送Minecraft富文本消息",
    description="向指定的Minecraft聊天发送富文本消息。LLM需要提供特定格式的JSON字符串。",
)
async def send_rich_text(_ctx: AgentCtx, chat_key: str, rich_text_json: str):
    """发送Minecraft富文本消息
    CAUTION: THIS METHOD IS USED FOR SENDING MESSAGES VIA MINECRAFT ADAPTER, NOT A MODEL CONTEXT PROTOCOL (MCP) SERVICE.
    Args:
        chat_key (str): Minecraft 服务器的会话标识 (例如 'minecraft-servername')。
        rich_text_json (str): 一个 JSON 字符串，表示一个消息段(segment)的列表。
            每个 segment 都是一个 JSON 对象，用于描述一段具有特定样式的文本及其交互行为。

    富文本 Segment JSON 对象支持的键:
        - `text` (str, 必选): 该段的文本内容。
        - `color` (str, 可选): 文本颜色。有效值是 TextColor 枚举的小写形式，例如:
          `"black"`, `"dark_blue"`, `"dark_green"`, `"dark_aqua"`, `"dark_red"`,
          `"dark_purple"`, `"gold"`, `"gray"`, `"dark_gray"`, `"blue"`, `"green"`,
          `"aqua"`, `"red"`, `"light_purple"`, `"yellow"`, `"white"`.
          默认为服务器的默认文本颜色。
        - `bold` (bool, 可选): 文本是否加粗。默认为 `false`。
        - `italic` (bool, 可选): 文本是否斜体。默认为 `false`。
        - `underlined` (bool, 可选): 文本是否有下划线。默认为 `false`。
        - `strikethrough` (bool, 可选): 文本是否有删除线。默认为 `false`。
        - `obfuscated` (bool, 可选): 文本是否混淆显示 (随机字符)。默认为 `false`。
        - `insertion` (str, 可选): 当玩家在聊天中点击这段文本时，插入到玩家聊天输入框的文本。
        - `click_event_action` (str, 可选): 点击事件的动作。必须与 `click_event_value` 一起使用。
          有效值 (ClickAction 枚举的小写形式):
            - `"open_url"`: 打开一个 URL。
            - `"open_file"`: 打开一个文件 (通常在客户端上，不推荐用于服务器消息)。
            - `"run_command"`: 以玩家身份执行一条命令。
            - `"suggest_command"`: 在玩家聊天输入框中建议一条命令 (玩家需手动发送)。
            - `"change_page"`: (仅用于书本中) 改变书本的页面。
            - `"copy_to_clipboard"`: 将指定文本复制到玩家的剪贴板。
        - `click_event_value` (str, 可选): 点击事件的值。根据 `click_event_action` 的不同而不同:
            - 对于 `"open_url"`, `"suggest_command"`, `"copy_to_clipboard"`: 对应的 URL、命令字符串或要复制的文本。
            - 对于 `"run_command"`: 要执行的命令 (通常需要以 `/` 开头)。
            - 对于 `"change_page"`: 页码。
        - `hover_event_action` (str, 可选): 鼠标悬停事件的动作。目前主要支持 `"show_text"`。
          必须与 `hover_event_text_segments` 一起使用。
          有效值 (HoverAction 枚举的小写形式):
            - `"show_text"`: 悬停时显示一段富文本。
            - `"show_item"`: (暂未完全支持通过此JSON格式配置) 悬停时显示物品信息。
            - `"show_entity"`: (暂未完全支持通过此JSON格式配置) 悬停时显示实体信息。
        - `hover_event_text_segments` (List[Dict], 可选): 当 `hover_event_action` 为 `"show_text"` 时使用。
          它本身是一个消息段 (segment) 对象的列表 (与顶层结构相同，但不建议嵌套过深的悬停事件)，
          用于定义鼠标悬停时显示的富文本内容。

    示例 JSON 字符串:
    ```json
    [
        {"text": "欢迎来到 "},
        {"text": "我的世界!", "color": "gold", "bold": true},
        {
            "text": " 点击这里访问官网",
            "color": "aqua",
            "underlined": true,
            "click_event_action": "open_url",
            "click_event_value": "https://www.minecraft.net"
        },
        {
            "text": " 或 ",
            "color": "gray"
        },
        {
            "text": "执行命令",
            "color": "light_purple",
            "italic": true,
            "click_event_action": "suggest_command",
            "click_event_value": "/help",
            "hover_event_action": "show_text",
            "hover_event_text_segments": [
                {"text": "点击后会在你的聊天框输入 ", "color": "yellow"},
                {"text": "/help", "color": "red", "bold": true}
            ]
        }
    ]
    ```
    注意: JSON 字符串在作为参数传递时，内部的双引号需要被正确转义，例如在Python字符串中表示为 `\"`。
    示例参数: `rich_text_json='[{"text": "你好", "color": "red"}, {"text": " 世界!", "bold": true}]'`
    """

    def _parse_and_validate_segments(json_str: str) -> List[MessageSegment]:
        """解析并验证富文本JSON字符串，返回MessageSegment列表。"""
        try:
            segments_data = json.loads(json_str)
        except json.JSONDecodeError as e:
            logger.error(f"解析富文本 JSON 失败: {e}", exc_info=True)
            raise ValueError(f"富文本 JSON 格式错误: {e}") from e

        if not isinstance(segments_data, list):
            raise ValueError("富文本 JSON 解析后必须是一个列表。")  # noqa: TRY004

        parsed_message_segments: List[MessageSegment] = []
        for seg_dict in segments_data:
            if isinstance(seg_dict, dict):
                mc_segment = _llm_segment_to_mc_message_segment(seg_dict)
                if mc_segment:
                    parsed_message_segments.append(mc_segment)
            else:
                logger.warning(
                    f"[MinecraftUtils] 富文本列表中的元素不是字典: {seg_dict}",
                )

        if not parsed_message_segments:
            raise ValueError("没有可发送的有效富文本段。")
        return parsed_message_segments

    if not isinstance(chat_key, str) or not chat_key.startswith("minecraft-"):
        raise ValueError("chat_key 无效，必须是 'minecraft-servername' 格式。")
    if not isinstance(rich_text_json, str) or not rich_text_json.strip():
        raise ValueError("富文本 JSON 字符串不能为空。")

    bot: Optional[Bot] = get_bot(chat_key)
    if not bot:
        raise ConnectionError(f"未能找到或连接到 Minecraft 服务器: {chat_key}")

    try:
        message_segments = _parse_and_validate_segments(rich_text_json)

        channel = await DBChatChannel.get_channel(chat_key)
        preset = await channel.get_preset()

        plain_text_parts = []
        for seg in message_segments:  # message_segments 是 _parse_and_validate_segments 的结果
            if hasattr(seg, "data") and isinstance(seg.data, dict) and "text" in seg.data:
                plain_text_parts.append(str(seg.data["text"]))
        plain_text_content = "".join(plain_text_parts)

        await message_service.push_bot_message(
            chat_key=chat_key,
            agent_messages=plain_text_content,  # 使用提取的纯文本
            db_chat_channel=channel,
        )
        message_segments.insert(0, MessageSegment.text(f"<{preset.name}>", color=TextColor.GREEN))
        message_to_send = Message(message_segments)
        await bot.send_msg(message=message_to_send)
        logger.info(f"Minecraft 富文本消息已发送到 {chat_key}")
    except ValueError as e:
        logger.error(f"处理富文本消息失败: {e}", exc_info=True)
        raise
    except Exception as e:
        logger.error(
            f"发送 Minecraft 富文本消息到 {chat_key} 失败: {e}",
            exc_info=True,
        )
        raise Exception(f"发送 Minecraft 富文本消息失败: {e}") from e


@plugin.mount_sandbox_method(
    SandboxMethodType.AGENT,
    name="批量执行Minecraft RCON命令",
    description="在指定的Minecraft服务器上批量执行RCON命令。",
)
async def execute_rcon_commands(
    _ctx: AgentCtx,
    chat_key: str,
    commands: List[str],
    continue_on_error: bool = False,
) -> str:
    """批量执行Minecraft RCON命令
    CAUTION: THIS METHOD IS USED FOR EXECUTING RCON COMMANDS VIA MINECRAFT ADAPTER, NOT A MODEL CONTEXT PROTOCOL (MCP) SERVICE.
    Args:
        chat_key (str): Minecraft 服务器的会话标识 (例如 'minecraft-servername')。
        commands (List[str]): 要执行的 RCON 命令列表。
        continue_on_error (bool, optional): 如果为 True，则在单个命令出错时继续执行后续命令，
                                        并将错误信息记录在对应命令的结果中。
                                        如果为 False (默认)，则在任何命令出错时立即停止，
                                        并返回错误信息及之前已成功执行的命令结果。

    Returns:
        str:
            RCON 命令的执行结果。
            如果 continue_on_error 为 True，返回所有命令结果的拼接字符串，错误会明确指出。
            如果 continue_on_error 为 False 且发生错误，则返回一个描述错误的字符串，包含出错命令和已成功的结果。
            如果 continue_on_error 为 False 且所有命令成功，则返回所有成功结果的拼接字符串。

    Examples:
        execute_rcon_commands(chat_key="minecraft-myserver", commands=["say Hello", "time set day"], continue_on_error=False)
        -> "Command 'say Hello': Hello\nCommand 'time set day': Set the time to 24000"

        execute_rcon_commands(chat_key="minecraft-myserver", commands=["say Hello", "invalid_command", "say World"], continue_on_error=False)
        -> "Error executing command 'invalid_command': <具体错误>\nPrevious results:\nCommand 'say Hello': Hello"

        execute_rcon_commands(chat_key="minecraft-myserver", commands=["say Hello", "invalid_command", "say World"], continue_on_error=True)
        -> "Command 'say Hello': Hello\nCommand 'invalid_command': Error - <具体错误>\nCommand 'say World': World"
    """
    if not isinstance(chat_key, str) or not chat_key.startswith("minecraft-"):
        raise ValueError("chat_key 无效，必须是 'minecraft-servername' 格式。")
    if not isinstance(commands, list) or not all(isinstance(cmd, str) and cmd.strip() for cmd in commands):
        raise ValueError("RCON 命令列表无效，必须是非空字符串的列表。")
    if not commands:
        raise ValueError("RCON 命令列表不能为空。")

    bot: Optional[Bot] = get_bot(chat_key)
    if not bot:
        # 以字符串形式返回错误，因为这是 Agent 类型方法
        return f"错误：未能找到或连接到 Minecraft 服务器: {chat_key}"

    if not bot.rcon:
        return f"错误：Minecraft 服务器 {chat_key} 未配置或未连接 RCON。"

    results_log: List[str] = []  # 用于记录每个命令的文本结果或错误

    for command in commands:
        try:
            response = await bot.call_api("send_rcon_cmd", command=command)

            response_msg = ""
            if isinstance(response, tuple) and len(response) > 0 and isinstance(response[0], str):
                response_msg = response[0].strip()
            elif isinstance(response, str):
                response_msg = response.strip()

            is_error = response_msg and any(response_msg.startswith(prefix) for prefix in RCON_ERROR_PREFIXES)

            if is_error:
                error_message = f"在 {chat_key} 上执行 RCON 命令 '{command}' 失败: {response_msg}"
                logger.error(error_message)
                if not continue_on_error:
                    error_response = f"执行命令 '{command}' 时出错: {response_msg}"
                    if results_log:  # 如果之前有成功的结果
                        error_response += "\n先前成功的结果:\n" + "\n".join(results_log)
                    return error_response
                # 如果 continue_on_error 为 True，则记录错误并继续
                results_log.append(f"Command '{command}': Error - {response_msg}")
            else:
                # 命令成功执行
                result_str = response_msg if response_msg else "指令已成功执行，无返回内容"
                log_entry = f"Command '{command}': {result_str}"
                results_log.append(log_entry)
                logger.info(
                    f"RCON 命令 '{command}' 在 {chat_key} 上执行成功，响应: {result_str}",
                )

        except Exception as e:
            # 此处捕获的是网络错误等更底层的异常
            error_message = f"在 {chat_key} 上执行 RCON 命令 '{command}' 时发生意外: {e}"
            logger.error(error_message, exc_info=True)
            if not continue_on_error:
                error_response = f"执行命令 '{command}' 时出错: {e}"
                if results_log:  # 如果之前有成功的结果
                    error_response += "\n先前成功的结果:\n" + "\n".join(results_log)
                return error_response
            # 如果 continue_on_error 为 True，则记录错误并继续
            results_log.append(f"Command '{command}': Error - {e}")

    return "\n".join(results_log)



import asyncio
import json
from typing import Any, Callable, Coroutine, Dict, List, Optional

import aiohttp
from pydantic import BaseModel, Field, ValidationError

from nekro_agent.core.logger import logger


class Danmaku(BaseModel):
    """弹幕消息"""

    from_live_room: str = Field(default="", description="消息来源(房间号)")
    uid: str = Field(default="0", description="消息用户ID")
    username: str = Field(default="unknown", description="用户名")
    text: str = Field(default="", description="弹幕内容")
    time: int = Field(default=0, description="弹幕发送时间")
    url: List[str] = Field(default_factory=list, description="弹幕中的表情图片url列表")
    is_trigger: bool = Field(default=False, description="是否触发LLM (由ws客户端接收并处理)")
    is_system: bool = Field(default=False, description="是否作为system身份发送 (由ws客户端接收并处理)")


class BilibiliWebSocketClient:
    """
    Bilibili WebSocket 客户端

    管理与 `/ws/danmaku` 和 `/ws/animate_control` 的连接。
    """

    def __init__(
        self,
        base_url: str,
        danmaku_handler: Callable[["BilibiliWebSocketClient", Danmaku], Coroutine[Any, Any, None]],
    ):
        self._base_url = base_url.rstrip("/")
        self._danmaku_handler = danmaku_handler
        self._danmaku_url = f"{self._base_url}/ws/danmaku"
        self._animate_url = f"{self._base_url}/ws/animate_control"

        self._session: Optional[aiohttp.ClientSession] = None
        self._danmaku_ws: Optional[aiohttp.ClientWebSocketResponse] = None
        self._animate_ws: Optional[aiohttp.ClientWebSocketResponse] = None

        self._main_tasks: List[asyncio.Task] = []
        self._is_closing = False

        # For animate_control request-response
        self._animate_lock = asyncio.Lock()
        self._pending_animate_future: Optional[asyncio.Future] = None

    async def start_with_auto_reconnect(self) -> None:
        """启动客户端并保持自动重连"""
        self._session = aiohttp.ClientSession()
        self._is_closing = False

        danmaku_task = asyncio.create_task(
            self._connection_loop(self._danmaku_url, self._on_danmaku_connect, "Danmaku"),
        )
        animate_task = asyncio.create_task(
            self._connection_loop(self._animate_url, self._on_animate_connect, "Animate"),
        )
        self._main_tasks = [danmaku_task, animate_task]

        await asyncio.gather(*self._main_tasks, return_exceptions=True)

    async def _connection_loop(
        self,
        url: str,
        on_connect: Callable[[aiohttp.ClientWebSocketResponse], Coroutine[Any, Any, None]],
        name: str,
    ) -> None:
        """通用连接循环，包含重连逻辑"""
        backoff_delay = 1
        while not self._is_closing:
            try:
                logger.info(f"正在连接到 {name} WebSocket: {url}")
                if not self._session or self._session.closed:
                    self._session = aiohttp.ClientSession()

                async with self._session.ws_connect(url, heartbeat=30) as ws:
                    logger.info(f"{name} WebSocket 连接成功: {url}")
                    backoff_delay = 1 
                    await on_connect(ws)
            except aiohttp.ClientConnectorError as e:
                logger.warning(f"{name} WebSocket 连接失败: {e}")
            except aiohttp.WSServerHandshakeError as e:
                logger.warning(f"{name} WebSocket握手失败: {e}")
            except Exception as e:
                logger.error(f"{name} WebSocket 循环遇到未知错误: {e}", exc_info=True)
            finally:
                # Fail any pending requests on this connection
                if name == "Animate" and self._pending_animate_future and not self._pending_animate_future.done():
                    self._pending_animate_future.set_exception(ConnectionError(f"{name} WebSocket disconnected"))

                if not self._is_closing:
                    logger.info(f"{name} WebSocket 将在 {backoff_delay} 秒后尝试重连...")
                    await asyncio.sleep(backoff_delay)
                    backoff_delay = min(backoff_delay * 2, 60)  

    async def _on_danmaku_connect(self, ws: aiohttp.ClientWebSocketResponse) -> None:
        self._danmaku_ws = ws
        await self._danmaku_receiver(ws)

    async def _on_animate_connect(self, ws: aiohttp.ClientWebSocketResponse) -> None:
        self._animate_ws = ws
        await self._animate_receiver(ws)

    async def _danmaku_receiver(self, ws: aiohttp.ClientWebSocketResponse) -> None:
        """接收并处理弹幕消息"""
        async for msg in ws:
            if self._is_closing:
                break
            if msg.type == aiohttp.WSMsgType.TEXT:
                try:
                    # 心跳 pong 不需要处理, aiohttp 会自动处理
                    danmaku = Danmaku.model_validate_json(msg.data)
                    await self._danmaku_handler(self, danmaku)
                except ValidationError as e:
                    logger.warning(f"解析弹幕消息失败: {e}, 原始数据: {msg.data}")
                except Exception as e:
                    logger.error(f"处理弹幕消息时发生错误: {e}", exc_info=True)
            elif msg.type == aiohttp.WSMsgType.ERROR:
                logger.error(f"Danmaku WebSocket 出现错误: {ws.exception()}")
                break
        logger.info("Danmaku WebSocket 接收循环结束.")

    async def _animate_receiver(self, ws: aiohttp.ClientWebSocketResponse) -> None:
        """接收并处理 animate control 的响应"""
        async for msg in ws:
            if self._is_closing:
                break
            if msg.type == aiohttp.WSMsgType.TEXT:
                logger.info(f"收到 Animate 消息: {msg.data}")
                if self._pending_animate_future and not self._pending_animate_future.done():
                    try:
                        data = json.loads(msg.data)
                        self._pending_animate_future.set_result(data)
                    except json.JSONDecodeError as e:
                        self._pending_animate_future.set_exception(e)
                else:
                    logger.warning(f"收到未请求的 Animate 消息: {msg.data}")
            elif msg.type == aiohttp.WSMsgType.ERROR:
                logger.error(f"Animate WebSocket 出现错误: {ws.exception()}")
                if self._pending_animate_future and not self._pending_animate_future.done():
                    self._pending_animate_future.set_exception(ws.exception() or ConnectionError("Animate WebSocket disconnected"))
                break
        logger.info("Animate WebSocket 接收循环结束.")

    async def send_animate_command(self, command: Dict[str, Any]) -> Dict[str, Any]:
        """
        向 /ws/animate_control 发送命令并等待响应。
        此方法不是线程安全的，也不支持并发请求。
        """
        async with self._animate_lock:
            if not self._animate_ws or self._animate_ws.closed:
                raise ConnectionError("Animate WebSocket 未连接。")

            loop = asyncio.get_running_loop()
            self._pending_animate_future = loop.create_future()

            try:
                logger.info(f"发送 Animate 命令: {command}")
                await self._animate_ws.send_json(command)
                return await asyncio.wait_for(self._pending_animate_future, timeout=30.0)
            except asyncio.TimeoutError:
                raise TimeoutError("Animate command timed out.") from None
            finally:
                self._pending_animate_future = None

    async def close(self) -> None:
        """关闭所有连接和任务"""
        if self._is_closing:
            return

        logger.info("正在关闭 Bilibili WebSocket 客户端...")
        self._is_closing = True

        for task in self._main_tasks:
            task.cancel()

        if self._danmaku_ws:
            await self._danmaku_ws.close()
        if self._animate_ws:
            await self._animate_ws.close()

        # Wait for session to close gracefully
        if self._session:
            await self._session.close()

        if self._main_tasks:
            await asyncio.gather(*self._main_tasks, return_exceptions=True)

        logger.info("Bilibili WebSocket 客户端已关闭。")


from typing import Optional

from nonebot import get_bots
from nonebot.adapters.minecraft import Bot

from nekro_agent.core import logger


def get_bot(identifier: Optional[str] = None) -> Optional[Bot]:
    """
    获取 Minecraft Bot 实例。

    Args:
        identifier: Bot 的标识符。
                    来自 nekro-agent 的 chat_key (例如 'minecraft-servername')，
                    其中 'servername' 应该对应 NoneBot 中 Bot 的 self_id。
                    如果为 None 或无法从中解析出 servername，则尝试返回第一个可用的 Minecraft Bot。
    """
    bots = get_bots()  # get_bots() 返回一个字典 {bot_id: Bot_instance}

    target_bot_id: Optional[str] = None
    if identifier and identifier.startswith("minecraft-"):
        parts = identifier.split("-", 1)
        if len(parts) > 1 and parts[1]:
            target_bot_id = parts[1]

    if target_bot_id:
        bot = bots.get(target_bot_id)
        if bot and isinstance(bot, Bot):
            return bot

    return None



import asyncio
import contextlib
import time
import uuid
from datetime import datetime, timedelta
from typing import Dict, List, Optional, Set, Union

from fastapi import Request
from sse_starlette.sse import EventSourceResponse

from nekro_agent.core.logger import logger


class ClientInfo:
    """客户端信息"""

    def __init__(self, client_id: str, name: str = "", platform: str = "unknown"):
        self.client_id = client_id
        self.name = name
        self.platform = platform  # 客户端平台标识
        self.connected_at = datetime.now()
        self.last_heartbeat = datetime.now()
        self.channels: Set[str] = set()  # 客户端关联的聊天频道
        self.event_queue: asyncio.Queue = asyncio.Queue()
        self.is_alive = True

    def update_heartbeat(self):
        """更新心跳时间"""
        self.last_heartbeat = datetime.now()

    def is_expired(self, timeout_seconds: int = 60) -> bool:
        """检查客户端是否超时"""
        return (datetime.now() - self.last_heartbeat).total_seconds() > timeout_seconds

    async def send_event(self, event_type: str, data: dict):
        """发送事件到客户端"""
        await self.event_queue.put({"type": event_type, "data": data})

    def add_channel(self, chat_key: str):
        """添加客户端关联的聊天频道"""
        self.channels.add(chat_key)

    def remove_channel(self, chat_key: str):
        """移除客户端关联的聊天频道"""
        self.channels.discard(chat_key)


class ClientManager:
    """客户端管理器"""

    def __init__(self):
        self.clients: Dict[str, ClientInfo] = {}
        self.cleanup_task = None

    async def start(self):
        """启动客户端管理器"""
        if self.cleanup_task is None:
            self.cleanup_task = asyncio.create_task(self._cleanup_loop())
            logger.info("SSE 客户端管理器已启动")

    async def stop(self):
        """停止客户端管理器"""
        if self.cleanup_task:
            self.cleanup_task.cancel()
            with contextlib.suppress(asyncio.CancelledError):
                await self.cleanup_task
            self.cleanup_task = None
            logger.info("SSE 客户端管理器已停止")

    async def _cleanup_loop(self):
        """定期清理失效客户端"""
        while True:
            try:
                await asyncio.sleep(30)
                expired_clients = []

                for client_id, client in self.clients.items():
                    if client.is_expired():
                        expired_clients.append(client_id)

                for client_id in expired_clients:
                    logger.info(f"SSE 客户端 {client_id} 心跳超时，移除连接")
                    client = self.clients.pop(client_id, None)
                    if client:
                        client.is_alive = False

            except asyncio.CancelledError:
                break
            except Exception as e:
                logger.error(f"SSE 客户端清理异常: {e}")

    def register_client(self, name: str = "", platform: str = "unknown") -> ClientInfo:
        """注册新客户端

        Args:
            name: 客户端名称
            platform: 平台标识

        Returns:
            ClientInfo: 客户端信息
        """
        client_id = str(uuid.uuid4())
        client = ClientInfo(client_id, name, platform)
        self.clients[client_id] = client
        logger.info(f"SSE 客户端 {client_id} ({name}/{platform}) 已连接")
        return client

    def unregister_client(self, client_id: str):
        """注销客户端"""
        client = self.clients.pop(client_id, None)
        if client:
            client.is_alive = False
            logger.info(f"SSE 客户端 {client_id} ({client.name}) 已断开连接")

    def get_client(self, client_id: str) -> Optional[ClientInfo]:
        """获取客户端信息"""
        return self.clients.get(client_id)

    def get_clients_by_channel(self, chat_key: str) -> List[ClientInfo]:
        """获取指定频道的所有客户端"""
        return [client for client in self.clients.values() if chat_key in client.channels]

    async def broadcast_to_channel(self, chat_key: str, event_type: str, data: dict):
        """向指定频道的所有客户端广播事件"""
        clients = self.get_clients_by_channel(chat_key)
        for client in clients:
            await client.send_event(event_type, data)

    async def broadcast_to_all(self, event_type: str, data: dict):
        """向所有客户端广播事件"""
        for client in self.clients.values():
            await client.send_event(event_type, data)

    def get_client_by_name(self, name: str) -> Optional[ClientInfo]:
        """根据名称获取客户端信息

        Args:
            name: 客户端名称

        Returns:
            Optional[ClientInfo]: 客户端信息，如果不存在则返回None
        """
        for client in self.clients.values():
            if client.name == name:
                return client
        return None


# 全局客户端管理器实例
client_manager = ClientManager()


async def sse_stream(request: Request, client: ClientInfo):
    """SSE 事件流生成器"""
    try:
        # 发送连接成功事件
        yield {"event": "connected", "data": {"client_id": client.client_id, "timestamp": int(time.time())}}

        # 心跳计时器
        heartbeat_timer = 0

        while client.is_alive:
            # 每5秒发送一次心跳
            if time.time() - heartbeat_timer >= 5:
                yield {"event": "heartbeat", "data": {"timestamp": int(time.time())}}
                heartbeat_timer = time.time()
                client.update_heartbeat()

            # 尝试从队列获取事件，最多等待1秒
            try:
                event = await asyncio.wait_for(client.event_queue.get(), timeout=1.0)
                yield {"event": event["type"], "data": event["data"]}
                client.event_queue.task_done()
            except asyncio.TimeoutError:
                pass  # 无新事件，继续循环

            # 检查客户端请求是否已断开
            if await request.is_disconnected():
                logger.info(f"SSE 客户端 {client.client_id} 连接已断开")
                break

    except asyncio.CancelledError:
        logger.info(f"SSE 客户端 {client.client_id} SSE流已取消")
    except Exception as e:
        logger.error(f"SSE 客户端 {client.client_id} SSE流异常: {e}")
    finally:
        # 确保客户端被正确注销
        client_manager.unregister_client(client.client_id)

    logger.info(f"SSE 客户端 {client.client_id} SSE流已结束")


def create_sse_response(request: Request, client: Union[ClientInfo, str]) -> EventSourceResponse:
    """创建SSE响应

    Args:
        request: 请求对象
        client: 客户端实例或客户端名称
    """
    # 如果是字符串，则创建新客户端
    if isinstance(client, str):
        client = client_manager.register_client(client)

    return EventSourceResponse(sse_stream(request, client))



from pydantic import Field

from nekro_agent.adapters.interface.base import BaseAdapterConfig
from nekro_agent.core.core_utils import ExtraField
from typing import Optional

class WeChatPadConfig(BaseAdapterConfig):
    """WeChatPad 适配器配置"""

    WECHATPAD_API_URL: str = Field(
        default="http://127.0.0.1:8080",
        title="WeChatPadPro API 地址",
        description="您部署的 WeChatPadPro 项目的 API 服务地址",
    )

    WECHATPAD_AUTH_KEY: str = Field(
        default="",
        title="WeChatPadPro AuthKey",
        description="用于 API 认证的 AuthKey，作为 query parameter 'key' 传递",
        json_schema_extra=ExtraField(required=True).model_dump(),
    )

    WECHATPAD_CALLBACK_URL: str = Field(
        default="",
        title="WeChatPadPro 回调地址",
        description="用于接收微信事件（如新消息）的回调地址。需要是一个公网可访问的 URL，例如: http://your_domain.com/api/v1/adapters/wechatpad/webhook",
    )

    SESSION_PROCESSING_WITH_EMOJI: bool = Field(
        default=False,
        title="显示处理中表情反馈",
        description="当 AI 开始处理消息时，对应消息会显示处理中表情反馈（微信不支持此功能）",
    )



"""
SSE 服务层
=========

提供高层业务逻辑实现，封装与客户端的交互。

主要功能:
1. 消息发送
2. 用户信息获取
3. 频道信息获取
4. 消息反应设置
5. 大文件分块推送（解决客户端接收时的"chunk too big"问题）

注意：分块传输是服务端向客户端推送大文件时使用，避免SSE事件过大导致客户端接收错误。
"""

import asyncio
import math
import uuid
from typing import Any, Dict, List, Optional

from pydantic import BaseModel

from nekro_agent.adapters.sse.sdk.models import (
    ChannelInfo,
    ChunkComplete,
    ChunkData,
    Event,
    GetChannelInfoRequest,
    GetSelfInfoRequest,
    GetUserInfoRequest,
    Request,
    RequestType,
    SendMessage,
    SetMessageReactionRequest,
    SetMessageReactionResponse,
    UserInfo,
)
from nekro_agent.core import logger

from .client import SseClient, SseClientManager

# 分块大小配置
CHUNK_SIZE = 64 * 1024  # 64KB per chunk
MAX_BASE64_SIZE = 1 * 1024 * 1024  # 1MB，超过此大小进行分块传输


class SseApiService:
    """SSE API 服务

    封装与客户端的交互逻辑
    """

    def __init__(self, client_manager: SseClientManager):
        """初始化服务

        Args:
            client_manager: 客户端管理器
        """
        self.client_manager = client_manager

    def _should_use_chunked_transfer(self, data: str) -> bool:
        """判断是否应该使用分块传输

        Args:
            data: base64编码的数据

        Returns:
            bool: 是否使用分块传输
        """
        # 估算base64数据的原始大小（base64比原始数据大约33%）
        estimated_size = len(data) * 3 // 4
        return estimated_size > MAX_BASE64_SIZE

    async def _send_chunked_data(self, client: SseClient, data: str, mime_type: str, filename: str, file_type: str) -> bool:
        """分块发送大数据

        Args:
            client: 目标客户端
            data: base64编码的数据
            mime_type: MIME类型
            filename: 文件名
            file_type: 文件类型 (image/file)

        Returns:
            bool: 是否发送成功
        """
        chunk_id = str(uuid.uuid4())
        total_size = len(data) * 3 // 4  # 估算原始大小
        total_chunks = math.ceil(len(data) / CHUNK_SIZE)

        logger.info(f"开始分块传输: {filename}, 大小: {total_size} bytes, 分块数: {total_chunks}")

        try:
            # 发送所有分块
            for i in range(total_chunks):
                start_pos = i * CHUNK_SIZE
                end_pos = min(start_pos + CHUNK_SIZE, len(data))
                chunk_data = data[start_pos:end_pos]

                chunk_event = Event(
                    event=RequestType.FILE_CHUNK.value,
                    data=ChunkData(
                        chunk_id=chunk_id,
                        chunk_index=i,
                        total_chunks=total_chunks,
                        chunk_data=chunk_data,
                        chunk_size=len(chunk_data),
                        total_size=total_size,
                        mime_type=mime_type,
                        filename=filename,
                        file_type=file_type,
                    ),
                )

                await client.send_event(chunk_event)
                logger.debug(f"发送分块 {i+1}/{total_chunks}: {len(chunk_data)} bytes")

                # 稍微延迟以避免过快发送导致客户端处理不过来
                await asyncio.sleep(0.01)

            # 发送传输完成事件
            complete_event = Event(
                event=RequestType.FILE_CHUNK_COMPLETE.value,
                data=ChunkComplete(chunk_id=chunk_id, success=True, message=f"文件 {filename} 传输完成"),
            )
            await client.send_event(complete_event)

        except Exception as e:
            logger.error(f"分块传输失败: {filename}, 错误: {e}")

            # 发送传输失败事件
            try:
                error_event = Event(
                    event=RequestType.FILE_CHUNK_COMPLETE.value,
                    data=ChunkComplete(chunk_id=chunk_id, success=False, message=f"文件 {filename} 传输失败: {e!s}"),
                )
                await client.send_event(error_event)
            except Exception:
                pass  # 忽略发送错误事件的异常

            return False
        else:
            logger.success(f"分块传输完成: {filename}")
            return True

    async def send_message_to_clients(self, clients: List[SseClient], message: SendMessage) -> bool:
        """向客户端列表发送消息

        Args:
            clients: 客户端列表
            message: 要发送的消息

        Returns:
            bool: 是否成功发送
        """
        if not clients:
            logger.warning("没有可用的SSE客户端")
            return False

        # 检查消息段是否包含大文件，直接分块发送
        has_large_files = await self._process_large_files(message, clients)

        # 如果包含大文件，直接返回成功（文件已通过分块方式发送）
        if has_large_files:
            logger.info("消息包含大文件，已通过分块方式发送")
            return True

        # 普通消息正常发送
        return await self._send_normal_message(clients, message)

    async def _send_normal_message(self, clients: List[SseClient], message: SendMessage) -> bool:
        """发送普通消息（不包含大文件）

        Args:
            clients: 客户端列表
            message: 要发送的消息

        Returns:
            bool: 是否成功发送
        """
        # 构建请求
        request_id = str(uuid.uuid4())

        for client in clients:
            # 创建请求和响应等待对象
            response_future = asyncio.Future()

            # 注册响应处理器
            async def handle_response(response_data: BaseModel, future=response_future) -> bool:
                # 使用字典转换访问属性，避免类型检查问题
                data_dict = response_data.dict() if hasattr(response_data, "dict") else {}
                success = data_dict.get("success", False)
                future.set_result(success)
                return True

            client.register_handler(request_id, handle_response)

            try:
                # 发送消息请求
                await client.send_event(
                    Event(
                        event=RequestType.SEND_MESSAGE.value,
                        data=Request(
                            request_id=request_id,
                            data={
                                "channel_id": message.channel_id,
                                "segments": [segment.model_dump() for segment in message.segments],
                            },
                        ),
                    ),
                )

                # 等待响应，设置超时
                try:
                    result = await asyncio.wait_for(response_future, timeout=15)
                    if result:
                        return True
                except asyncio.TimeoutError:
                    logger.warning(f"等待客户端 {client.client_id} 响应超时")
                    continue

            except Exception as e:
                logger.error(f"向客户端 {client.client_id} 发送消息异常: {e}")
                continue

        # 所有客户端均发送失败
        return False

    async def _process_large_files(self, message: SendMessage, clients: List[SseClient]) -> bool:
        """检查并处理消息中的大文件，对大文件进行分块传输

        Args:
            message: 原始消息
            clients: 目标客户端列表

        Returns:
            bool: 是否包含大文件（已分块发送）
        """
        has_large_files = False

        for segment in message.segments:
            segment_dict = segment.model_dump()
            segment_type = segment_dict.get("type")

            # 检查是否是图片或文件段，且包含大的base64数据
            if segment_type in ["image", "file"]:
                base64_url = segment_dict.get("base64_url", "")

                if base64_url and base64_url.startswith("data:"):
                    # 提取base64数据部分
                    try:
                        header, data = base64_url.split(",", 1)
                        mime_type = header.split(";")[0].split(":")[1] if ":" in header else "application/octet-stream"

                        if self._should_use_chunked_transfer(data):
                            # 需要分块传输
                            filename = segment_dict.get("name", f"file_{uuid.uuid4().hex[:8]}")
                            has_large_files = True

                            logger.info(f"检测到大文件 {filename}，开始分块传输")

                            # 向所有客户端分块发送文件
                            for client in clients:
                                success = await self._send_chunked_data(
                                    client,
                                    data,
                                    mime_type,
                                    filename,
                                    segment_type or "file",  # 确保不是None
                                )
                                if not success:
                                    logger.error(f"向客户端 {client.client_id} 分块发送文件失败")

                    except Exception as e:
                        logger.error(f"处理大文件失败: {e}")

        return has_large_files

    async def _request_from_client(
        self,
        request_type: RequestType,
        request_data: BaseModel,
        timeout: float = 10.0,
    ) -> Optional[BaseModel]:
        """向任一可用客户端发送请求

        Args:
            request_type: 请求类型
            request_data: 请求数据（BaseModel对象）
            timeout: 超时时间（秒）

        Returns:
            Optional[BaseModel]: 响应数据，失败则返回None
        """
        # 获取任一可用客户端
        clients = list(self.client_manager.clients.values())
        if not clients:
            logger.warning("没有可用的SSE客户端")
            return None

        client = clients[0]
        request_id = str(uuid.uuid4())

        # 创建请求和响应等待对象
        response_future = asyncio.Future()

        # 注册响应处理器
        async def handle_response(response_data: BaseModel, future=response_future) -> bool:
            # 使用字典转换访问属性，避免类型检查问题
            data_dict = response_data.dict() if hasattr(response_data, "dict") else {}
            success = data_dict.get("success", False)

            if success:
                # 直接返回原始响应数据，让调用方进行类型转换
                future.set_result(response_data)
            else:
                future.set_result(None)
            return True

        client.register_handler(request_id, handle_response)

        try:
            # 发送请求
            await client.send_event(
                Event(
                    event=request_type.value,
                    data=Request(
                        request_id=request_id,
                        data=request_data.model_dump(),
                    ),
                ),
            )

            # 等待响应，设置超时
            return await asyncio.wait_for(response_future, timeout=timeout)
        except asyncio.TimeoutError:
            logger.warning(f"等待客户端 {client.client_id} 响应超时: {request_type}")
            return None
        except Exception as e:
            logger.error(f"向客户端 {client.client_id} 发送请求异常: {request_type}, {e}")
            return None

    async def get_self_info(self) -> Optional[UserInfo]:
        """获取机器人自身信息

        Returns:
            Optional[UserInfo]: 机器人信息，失败则返回None
        """
        response = await self._request_from_client(RequestType.GET_SELF_INFO, GetSelfInfoRequest())

        if not response:
            return None

        try:
            # 从响应中提取data字段并转换为UserInfo
            response_dict = response.dict() if hasattr(response, "dict") else {}
            user_data = response_dict.get("data", {})
            return UserInfo(**user_data)
        except Exception as e:
            logger.error(f"解析机器人信息失败: {e}")
            return None

    async def get_user_info(self, user_id: str) -> Optional[UserInfo]:
        """获取用户信息

        Args:
            user_id: 用户ID

        Returns:
            Optional[UserInfo]: 用户信息，失败则返回None
        """
        response = await self._request_from_client(RequestType.GET_USER_INFO, GetUserInfoRequest(user_id=user_id))

        if not response:
            return None

        try:
            # 从响应中提取data字段并转换为UserInfo
            response_dict = response.dict() if hasattr(response, "dict") else {}
            user_data = response_dict.get("data", {})
            return UserInfo(**user_data)
        except Exception as e:
            logger.error(f"解析用户信息失败: {e}")
            return None

    async def get_channel_info(self, channel_id: str) -> Optional[ChannelInfo]:
        """获取频道信息

        Args:
            channel_id: 频道ID

        Returns:
            Optional[ChannelInfo]: 频道信息，失败则返回None
        """
        response = await self._request_from_client(RequestType.GET_CHANNEL_INFO, GetChannelInfoRequest(channel_id=channel_id))

        if not response:
            return None

        try:
            # 从响应中提取data字段并转换为ChannelInfo
            response_dict = response.dict() if hasattr(response, "dict") else {}
            channel_data = response_dict.get("data", {})
            return ChannelInfo(**channel_data)
        except Exception as e:
            logger.error(f"解析频道信息失败: {e}")
            return None

    async def set_message_reaction(self, message_id: str, status: bool = True) -> bool:
        """设置消息反应

        Args:
            message_id: 消息ID
            status: 反应状态

        Returns:
            bool: 是否设置成功
        """
        response = await self._request_from_client(
            RequestType.SET_MESSAGE_REACTION,
            SetMessageReactionRequest(message_id=message_id, status=status),
        )

        if not response:
            return False

        try:
            # 从响应中提取data字段并转换为SetMessageReactionResponse
            response_dict = response.dict() if hasattr(response, "dict") else {}
            reaction_data = response_dict.get("data", {})
            reaction_response = SetMessageReactionResponse(**reaction_data)
        except Exception as e:
            logger.error(f"解析消息反应设置结果失败: {e}")
            return False
        else:
            return reaction_response.success



import hashlib
import time
from typing import Optional, Union

from nonebot import on_message
from nonebot.adapters.minecraft import Bot, Event
from nonebot.adapters.minecraft.event.base import (
    BaseChatEvent,
    BaseDeathEvent,
    BasePlayerCommandEvent,
)
from nonebot.matcher import Matcher

from nekro_agent.adapters.interface import (
    BaseAdapter,
    PlatformChannel,
    PlatformMessage,
    PlatformUser,
    collect_message,
)
from nekro_agent.core import config, logger
from nekro_agent.schemas.chat_message import (
    ChatMessageSegment,
    ChatMessageSegmentType,
    ChatType,
)


def register_matcher(adapter: BaseAdapter):
    @on_message(priority=999, block=False).handle()
    async def _(_matcher: Matcher, event: Union[BaseChatEvent, BasePlayerCommandEvent, BaseDeathEvent], _bot: Bot):
        """Minecraft 消息收集与预处理"""

        plt_channel: PlatformChannel = PlatformChannel(
            channel_id=event.server_name,
            channel_name=event.server_name,
            channel_type=ChatType.GROUP,
        )

        sender_name: Optional[str] = event.player.nickname
        original_player_uuid = str(event.player.uuid)
        short_player_id = hashlib.sha1(original_player_uuid.encode("utf-8")).hexdigest()[:10]

        plt_user: PlatformUser = PlatformUser(
            platform_name="Minecraft",
            user_id=short_player_id,
            user_name=sender_name,
            user_avatar="",
        )

        msg_list = []
        plain_text_content = event.message.extract_plain_text()
        msg_list.append(
            ChatMessageSegment(
                type=ChatMessageSegmentType.TEXT,
                text=plain_text_content,
            ),
        )
        logger.info(f"Minecraft Message:{plain_text_content}")
        msg_id_from_event = event.message_id
        if msg_id_from_event and msg_id_from_event.strip():
            msg_id = msg_id_from_event
        else:
            msg_id = f"mc_{event.server_name}_{short_player_id}_{int(event.timestamp)}"

        plt_msg: PlatformMessage = PlatformMessage(
            message_id=msg_id,
            sender_id=short_player_id,
            sender_name=sender_name,
            sender_nickname=sender_name,
            content_data=msg_list,
            content_text=event.message.extract_plain_text(),
            is_tome=False,
            timestamp=int(event.timestamp),
        )

        await collect_message(adapter, plt_channel, plt_user, plt_msg)



"""
SSE 命令处理模块
==============

定义SSE协议使用的各种命令处理逻辑，包含:
1. 命令模型 (Command, RegisterCommand, SubscribeCommand等)
2. 命令处理器 (handle_register, handle_subscribe等)
3. 辅助函数和工具
"""

from functools import wraps
from typing import Any, Callable, Dict, Optional, Type

from fastapi import HTTPException
from pydantic import BaseModel, Field

from nekro_agent.adapters.interface import collect_message
from nekro_agent.adapters.interface.schemas.platform import (
    PlatformChannel,
    PlatformUser,
)
from nekro_agent.adapters.sse.core.client import SseClientManager
from nekro_agent.adapters.sse.core.message import SseMessageConverter
from nekro_agent.adapters.sse.sdk.models import (
    ChannelSubscribeRequest,
    ChannelSubscribeResponse,
    ClientCommand,
    ReceiveMessage,
    RegisterRequest,
    RegisterResponse,
)
from nekro_agent.adapters.sse.sdk.models import (
    Response as SseResponse,
)
from nekro_agent.adapters.utils import adapter_utils
from nekro_agent.core import logger
from nekro_agent.schemas.chat_message import ChatType

# 全局客户端管理器 - 从外部导入实例
client_manager: SseClientManager

# 消息转换器
message_converter = SseMessageConverter()

# 命令处理器注册表
_command_handlers: Dict[str, Callable] = {}

# 命令模型注册表
_command_models: Dict[str, Type[BaseModel]] = {}


# 装饰器：命令处理器
def command_handler(cmd: ClientCommand, model: Optional[Type[BaseModel]] = None):
    """命令处理器装饰器

    Args:
        cmd: 命令名称
        model: 命令数据模型（可选）
    """

    def decorator(func: Callable):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            return await func(*args, **kwargs)

        # 注册命令处理器
        _command_handlers[cmd.value] = wrapper

        # 注册命令模型
        if model:
            _command_models[cmd.value] = model

        return wrapper

    return decorator


# 消息命令模型
class MessageCommand(BaseModel):
    """消息命令"""

    channel_id: str = Field(..., description="频道ID")
    message: ReceiveMessage = Field(..., description="消息内容")


# 消息响应模型
class MessageResponse(BaseModel):
    """消息响应"""

    message: str = Field(..., description="响应消息")


def set_client_manager(manager: SseClientManager) -> None:
    """设置全局客户端管理器

    Args:
        manager: 客户端管理器实例
    """
    global client_manager
    client_manager = manager


# 注册命令处理器
@command_handler(ClientCommand.REGISTER, RegisterRequest)
async def handle_register(command: RegisterRequest) -> RegisterResponse:
    """处理注册命令"""
    client = client_manager.register_client(command.client_name, command.platform)
    return RegisterResponse(
        client_id=client.client_id,
        message=f"客户端 {command.client_name} ({command.client_version}) 注册成功",
    )


# 订阅频道命令处理器
@command_handler(ClientCommand.SUBSCRIBE, ChannelSubscribeRequest)
async def handle_subscribe(command: ChannelSubscribeRequest, client_id: str) -> ChannelSubscribeResponse:
    """处理订阅频道命令"""
    client = client_manager.get_client(client_id)
    if not client:
        raise HTTPException(status_code=404, detail=f"客户端 {client_id} 不存在")

    for channel_id in command.channel_ids:
        client.add_channel(channel_id)
    return ChannelSubscribeResponse(message=f"批量订阅 {len(command.channel_ids)} 个频道成功")


# 取消订阅频道命令处理器
@command_handler(ClientCommand.UNSUBSCRIBE, ChannelSubscribeRequest)
async def handle_unsubscribe(command: ChannelSubscribeRequest, client_id: str) -> ChannelSubscribeResponse:
    """处理取消订阅频道命令"""
    client = client_manager.get_client(client_id)
    if not client:
        raise HTTPException(status_code=404, detail=f"客户端 {client_id} 不存在")

    for channel_id in command.channel_ids:
        client.remove_channel(channel_id)
    return ChannelSubscribeResponse(message=f"批量取消订阅 {len(command.channel_ids)} 个频道成功")


# 消息命令处理器
@command_handler(ClientCommand.MESSAGE, MessageCommand)
async def handle_message(command: MessageCommand, client_id: str) -> MessageResponse:
    """处理消息命令"""
    client = client_manager.get_client(client_id)
    if not client:
        raise HTTPException(status_code=404, detail=f"客户端 {client_id} 不存在")

    # 消息已经由Pydantic模型自动解析
    sse_message = command.message

    # 创建平台消息
    platform_message = await message_converter.sse_to_platform_message(sse_message)

    platform_channel = PlatformChannel(
        channel_id=sse_message.channel_id,
        channel_name=sse_message.channel_name,
        channel_type=ChatType.GROUP,
    )

    # 创建用户信息
    platform_user = PlatformUser(
        platform_name=sse_message.platform_name,
        user_id=sse_message.from_id,
        user_name=sse_message.from_name,
        user_avatar="",
    )

    sse_adapter = adapter_utils.get_adapter("sse")

    # 调用消息收集器
    await collect_message(sse_adapter, platform_channel, platform_user, platform_message)

    return MessageResponse(message="消息接收成功")


# 响应命令处理器
@command_handler(ClientCommand.RESPONSE, SseResponse)
async def handle_response(command: SseResponse, client_id: str) -> Dict[str, bool]:
    """处理响应命令"""
    # 获取客户端
    client = client_manager.get_client(client_id)
    if not client:
        logger.error(f"客户端 {client_id} 不存在")
        return {"success": False}

    # 处理客户端响应
    result = await client.handle_response(command)

    return {"success": result}



"""
SSE 适配器
=========

该适配器实现了基于SSE (Server-Sent Events)的双向通信协议，
允许各种客户端通过HTTP与NekroAgent进行消息交换。

核心概念:
1. 适配器(Adapter): 负责协议与平台之间的转换
2. 客户端(Client): 连接到适配器的外部程序
3. 频道(Channel): 消息的交换场所，可以是群聊或私聊
4. 消息(Message): 在频道中交换的内容，包含各种消息段

适配器使用 channel_id 作为内部标识，格式为: `sse-{platform}-{channel_id}`
但对外部客户端，只暴露platform和channel_id概念。
"""

from typing import List, Optional, Type, overload

from fastapi import APIRouter
from pydantic import Field

from nekro_agent.adapters.interface.schemas.platform import (
    PlatformChannel,
    PlatformSendRequest,
    PlatformSendResponse,
    PlatformSendSegmentType,
    PlatformUser,
)
from nekro_agent.core import logger
from nekro_agent.schemas.chat_message import ChatType

from ..interface.base import AdapterMetadata, BaseAdapter, BaseAdapterConfig
from .commands import set_client_manager
from .core.client import SseClientManager
from .core.message import SseMessageConverter
from .core.service import SseApiService


class SSEConfig(BaseAdapterConfig):
    """SSE 适配器配置"""

    ACCESS_KEY: str = Field(
        default="",
        title="访问密钥",
        description="连接SSE服务所需的访问密钥，如果设置，客户端必须提供一致的密钥",
    )
    ALLOW_FILE_TRANSFER: bool = Field(default=False, title="是否允许文件传输", description="是否允许文件传输")
    MAX_FILE_SIZE: int = Field(default=10 * 1024 * 1024, title="最大文件大小（字节）", description="最大文件大小（字节）")
    ALLOWED_FILE_TYPES: List[str] = Field(
        default=["image/*", "application/*", "text/*"],
        title="允许的文件类型",
        description="允许的文件类型",
    )


class SSEAdapter(BaseAdapter[SSEConfig]):
    """SSE 协议适配器

    负责:
    1. 处理客户端连接与消息
    2. 转换消息格式
    3. 管理频道订阅
    """

    def __init__(self, config_cls: Type[SSEConfig] = SSEConfig):
        """初始化SSE适配器"""
        super().__init__(config_cls)
        # 核心组件
        self.client_manager = SseClientManager()
        self.message_converter = SseMessageConverter()
        self.service = SseApiService(self.client_manager)

        # 设置全局client_manager变量，确保commands.py中可以访问
        set_client_manager(self.client_manager)

    def get_adapter_router(self) -> APIRouter:
        from .routers import router, set_router_client_manager

        # 设置路由模块的client_manager
        set_router_client_manager(self.client_manager)

        return router

    async def init(self) -> None:
        """初始化适配器"""
        # 启动客户端管理器
        await self.client_manager.start()

        logger.info(f"SSE适配器 [{self.key}] 已初始化")

    async def cleanup(self) -> None:
        """清理适配器"""
        # 停止客户端管理器
        if hasattr(self, "client_manager") and self.client_manager:
            await self.client_manager.stop()
        return

    @property
    def key(self) -> str:
        """适配器唯一标识"""
        return "sse"

    @property
    def metadata(self) -> AdapterMetadata:
        return AdapterMetadata(
            name="SSE",
            description="基于 Server-Sent Events 的实时通信适配器，支持通用 HTTP 协议的多客户端接入",
            version="1.0.0",
            author="NekroAgent",
            homepage="https://github.com/nekro-agent/nekro-agent",
            tags=["sse", "http", "realtime", "api"],
        )

    @property
    def chat_key_rules(self) -> List[str]:
        """聊天标识规则说明"""
        return [
            "群聊: `sse-{channel_id}` (例如 sse-group_123456)",
            "私聊: `sse-{channel_id}` (例如 sse-private_user123)",
        ]

    def build_channel_id(self, platform: str, channel_id: str) -> str:
        """构建聊天标识

        Args:
            platform: 平台标识
            channel_id: 频道ID

        Returns:
            str: 聊天标识
        """
        return f"sse-{platform}-{channel_id}"

    async def forward_message(self, request: PlatformSendRequest) -> PlatformSendResponse:
        """推送消息到客户端

        Args:
            request: 包含要发送的消息段和目标channel_id

        Returns:
            PlatformSendResponse: 发送结果
        """
        try:
            # 解析channel_id
            _, channel_id = self.parse_chat_key(request.chat_key)

            # 检查是否有需要特殊处理的文件
            file_segments = [
                seg for seg in request.segments if seg.type in [PlatformSendSegmentType.FILE, PlatformSendSegmentType.IMAGE]
            ]

            # 如果存在文件但不允许文件传输，则返回错误
            if file_segments and not self.config.ALLOW_FILE_TRANSFER:
                logger.warning(f"禁止文件传输: {request.chat_key}")
                return PlatformSendResponse(success=False, error_message="文件传输已禁用")

            # 转换消息
            sse_message = await self.message_converter.platform_to_sse_message(
                channel_id=channel_id,
                segments=request.segments,
            )

            # 获取订阅该频道的客户端
            clients = self.client_manager.get_clients_by_channel(channel_id)
            if not clients:
                logger.warning(f"没有客户端处理频道 {channel_id}")
                return PlatformSendResponse(success=False, error_message=f"没有客户端处理频道 {channel_id}")

            # 发送消息
            response = await self.service.send_message_to_clients(clients=clients, message=sse_message)

            if response:
                return PlatformSendResponse(success=True)
            return PlatformSendResponse(success=False, error_message="所有客户端发送消息失败")

        except Exception as e:
            logger.error(f"发送SSE消息失败: {e}")
            return PlatformSendResponse(success=False, error_message=f"发送SSE消息失败: {e!s}")

    async def get_self_info(self) -> PlatformUser:
        """获取自身信息"""
        response = await self.service.get_self_info()

        if not response:
            return PlatformUser(user_id="", user_name="", platform_name="sse")

        return PlatformUser(
            user_id=response.user_id,
            user_name=response.user_name,
            user_avatar=response.user_avatar or "",
            platform_name=response.platform_name,
        )

    async def get_user_info(self, user_id: str, channel_id: str) -> PlatformUser:  # noqa: ARG002
        """获取用户信息"""
        response = await self.service.get_user_info(user_id)

        if not response:
            return PlatformUser(user_id=user_id, user_name=user_id, platform_name="sse")

        return PlatformUser(
            user_id=response.user_id,
            user_name=response.user_name,
            user_avatar=response.user_avatar or "",
            platform_name=response.platform_name,
        )

    async def get_channel_info(self, channel_id: str) -> PlatformChannel:
        """获取频道信息

        Args:
            channel_id: 频道ID，格式为 group_xxx 或 private_xxx

        Returns:
            PlatformChannel: 频道信息
        """
        # 判断频道类型
        channel_type = ChatType.GROUP if channel_id.startswith("group_") else ChatType.PRIVATE

        response = await self.service.get_channel_info(channel_id)

        if not response:
            return PlatformChannel(channel_id=channel_id, channel_name=channel_id, channel_type=channel_type)

        return PlatformChannel(
            channel_id=response.channel_id,
            channel_name=response.channel_name,
            channel_type=channel_type,
            channel_avatar=response.channel_avatar or "",
        )

    async def set_message_reaction(self, message_id: str, status: bool = True) -> bool:
        """设置消息反应"""
        return await self.service.set_message_reaction(message_id, status)



from pathlib import Path
from typing import List, Optional, Type

from fastapi import APIRouter
from nonebot.adapters.onebot.v11 import Bot, Message, MessageSegment
from pydantic import Field

from nekro_agent.adapters.interface.schemas.platform import (
    PlatformChannel,
    PlatformSendRequest,
    PlatformSendResponse,
    PlatformSendSegmentType,
    PlatformUser,
)
from nekro_agent.adapters.onebot_v11.matchers.message import register_matcher
from nekro_agent.core import config, logger
from nekro_agent.core.core_utils import ExtraField
from nekro_agent.core.os_env import OsEnv
from nekro_agent.models.db_chat_channel import DBChatChannel
from nekro_agent.schemas.chat_message import ChatType

from ..interface.base import AdapterMetadata, BaseAdapter, BaseAdapterConfig
from .core.bot import get_bot
from .tools.at_parser import SegAt, parse_at_from_text
from .tools.convertor import get_channel_type


class OnebotV11Config(BaseAdapterConfig):
    """Onebot V11 适配器配置"""

    BOT_QQ: str = Field(
        default="",
        title="机器人 QQ 号",
        json_schema_extra=ExtraField(required=True).model_dump(),
    )
    RESOLVE_CQ_CODE: bool = Field(
        default=False,
        title="是否解析 CQ 码",
        description="启用后，AI 发送的消息中的 CQ 码不再被视为纯文本，而是会被协议实现端解析为对应的富文本消息",
    )

    """NAPCAT 配置"""
    NAPCAT_ACCESS_URL: str = Field(
        default="http://127.0.0.1:6099/webui",
        title="NapCat WebUI 访问地址",
        description="NapCat 的 WebUI 地址，请确保对应端口已开放访问",
        json_schema_extra=ExtraField(placeholder="例: http://<服务器 IP>:<NapCat 端口>/webui").model_dump(),
    )
    NAPCAT_CONTAINER_NAME: str = Field(default="nekro_napcat", title="NapCat 容器名称")


class OnebotV11Adapter(BaseAdapter[OnebotV11Config]):
    """OneBot V11 适配器"""

    def __init__(self, config_cls: Type[OnebotV11Config] = OnebotV11Config):
        """初始化OnebotV11适配器"""
        super().__init__(config_cls)

    @property
    def key(self) -> str:
        return "onebot_v11"

    @property
    def metadata(self) -> AdapterMetadata:
        return AdapterMetadata(
            name="OneBot V11",
            description="OneBot V11 协议适配器，支持与兼容 OneBot V11 标准的 QQ 机器人实现进行通信",
            version="1.0.0",
            author="NekroAgent",
            homepage="https://github.com/nekro-agent/nekro-agent",
            tags=["qq", "onebot", "v11", "chat", "messaging"],
        )

    @property
    def chat_key_rules(self) -> List[str]:
        return [
            "群聊: `onebot_v11-group_123456` (123456为群号)",
            "私聊: `onebot_v11-private_123456` (123456为用户QQ号)",
        ]

    def get_adapter_router(self) -> APIRouter:
        """获取适配器路由"""
        from .routers import router

        return router

    async def init(self) -> None:
        """初始化适配器"""
        from . import matchers

        register_matcher(self)

    async def cleanup(self) -> None:
        """清理适配器"""
        return

    async def forward_message(self, request: PlatformSendRequest) -> PlatformSendResponse:
        """推送消息到 OneBot V11 协议端"""

        message_id: Optional[str] = None

        # 分离文件类型和其他类型的消息段
        file_segments = [seg for seg in request.segments if seg.type == PlatformSendSegmentType.FILE]
        other_segments = [seg for seg in request.segments if seg.type != PlatformSendSegmentType.FILE]

        # 先发送文件（如果有）
        if file_segments:
            await self._send_files(request.chat_key, file_segments)

        # 再发送其他类型消息（如果有）
        if other_segments:
            modified_request = PlatformSendRequest(
                chat_key=request.chat_key,
                segments=other_segments,
                ref_msg_id=request.ref_msg_id,
            )
            message_id = await self._send_message(modified_request)

        return PlatformSendResponse(success=True, message_id=message_id)

    async def _send_message(self, request: PlatformSendRequest) -> Optional[str]:
        """发送普通消息（文本、@、图片等）"""
        message = Message()

        if request.ref_msg_id:
            message.append(MessageSegment.reply(id_=int(request.ref_msg_id)))

        # 获取聊天频道信息用于 @ 解析
        db_chat_channel = await DBChatChannel.get_channel(chat_key=request.chat_key)

        for segment in request.segments:
            if segment.type == PlatformSendSegmentType.TEXT:
                if segment.content.strip():
                    # NoneBot 特有功能：解析文本中的 @ 信息
                    seg_data = await parse_at_from_text(segment.content, db_chat_channel)

                    for seg in seg_data:
                        if isinstance(seg, str):
                            if seg.strip():
                                message.append(MessageSegment.text(seg))
                        elif isinstance(seg, SegAt):  # SegAt 对象
                            message.append(MessageSegment.at(user_id=seg.platform_user_id))

            elif segment.type == PlatformSendSegmentType.AT:
                if segment.at_info:
                    message.append(MessageSegment.at(user_id=segment.at_info.platform_user_id))
            elif segment.type == PlatformSendSegmentType.IMAGE:
                # 图片以富文本形式发送
                if segment.file_path:
                    file_path = Path(segment.file_path)
                    if file_path.exists():
                        message.append(MessageSegment.image(file=file_path.read_bytes()))
                    else:
                        message.append(MessageSegment.text(f"Image file not found: {segment.file_path}"))
            else:
                logger.warning(f"Unsupported segment type in normal mode: {segment.type}")

        if message:
            return await self._send_to_chat(request.chat_key, message)
        return None

    async def _send_files(self, chat_key: str, file_segments: List) -> None:
        """发送文件（文件上传模式）"""
        bot: Bot = get_bot()
        files: List[Path] = []

        # 收集所有文件路径
        for segment in file_segments:
            if segment.file_path:
                file_path = Path(segment.file_path)
                if file_path.exists():
                    files.append(file_path)
                else:
                    logger.warning(f"File not found: {segment.file_path}")

        if not files:
            logger.warning("No valid files to send")
            return

        # 获取聊天频道信息
        db_chat_channel = await DBChatChannel.get_channel(chat_key=chat_key)
        chat_type = db_chat_channel.chat_type

        # 从channel_id中提取真实的ID
        chat_id = int(db_chat_channel.channel_id.split("_")[1])

        # 如果配置了 OneBot 服务器挂载目录，需要转换路径
        def get_onebot_path(file_path: Path) -> Path:
            if config.SANDBOX_ONEBOT_SERVER_MOUNT_DIR:
                return Path(config.SANDBOX_ONEBOT_SERVER_MOUNT_DIR) / file_path.relative_to(Path(OsEnv.DATA_DIR))
            return file_path

        if chat_type is ChatType.GROUP:
            for file in files:
                logger.info(f"Sending file: {file}")
                onebot_path = get_onebot_path(file)
                await bot.upload_group_file(
                    group_id=chat_id,
                    file=str(onebot_path),
                    name=file.name,
                )
        elif chat_type is ChatType.PRIVATE:
            for file in files:
                onebot_path = get_onebot_path(file)
                await bot.upload_private_file(
                    user_id=chat_id,
                    file=str(onebot_path),
                    name=file.name,
                )
        else:
            raise ValueError("Invalid chat type")

    async def _send_to_chat(self, chat_key: str, message: Message) -> str:
        """发送消息到指定聊天"""
        bot: Bot = get_bot()

        # 获取聊天频道信息
        db_chat_channel = await DBChatChannel.get_channel(chat_key=chat_key)
        chat_type = db_chat_channel.chat_type

        # 从channel_id中提取真实的ID
        chat_id = int(db_chat_channel.channel_id.split("_")[1])

        if chat_type is ChatType.GROUP:
            ret = await bot.send_group_msg(group_id=chat_id, message=message, auto_escape=self.config.RESOLVE_CQ_CODE)
        elif chat_type is ChatType.PRIVATE:
            ret = await bot.send_private_msg(user_id=chat_id, message=message, auto_escape=self.config.RESOLVE_CQ_CODE)
        else:
            raise ValueError("Invalid chat type")

        logger.debug(f"发送消息成功: {ret}")
        return str(ret.get("message_id", "")) or ""

    async def get_self_info(self) -> PlatformUser:
        """获取自身信息"""
        bot: Bot = get_bot()
        if bot:
            logger.info(f"Self_id:{bot.self_id} user_name:{bot.self_id}")
            return PlatformUser(platform_name="QQ", user_id=str(bot.self_id), user_name=bot.self_id)
        raise ValueError("No bot found")

    async def get_user_info(self, user_id: str, channel_id: str) -> PlatformUser:
        """获取用户(或者群聊用户)信息"""
        raise NotImplementedError

    async def get_channel_info(self, channel_id: str) -> PlatformChannel:
        """获取频道信息"""
        chat_type = get_channel_type(channel_id)
        if chat_type == ChatType.GROUP:
            try:
                channel_name = (await get_bot().get_group_info(group_id=int(channel_id.replace("group_", ""))))["group_name"]
            except Exception as e:
                logger.error(f"获取群组名称失败: {e!s}")
                channel_name = channel_id
        elif chat_type == ChatType.PRIVATE:
            channel_name = (await get_bot().get_stranger_info(user_id=int(channel_id.replace("private_", ""))))["nickname"]
        else:
            channel_name = channel_id

        return PlatformChannel(channel_id=channel_id, channel_name=channel_name, channel_type=chat_type)

    async def set_message_reaction(self, message_id: str, status: bool = True) -> bool:
        """设置消息反应（NoneBot 实现）

        Args:
            message_id (str): 消息ID
            status (bool): True为设置反应，False为取消反应

        Returns:
            bool: 是否成功设置
        """
        try:
            bot: Bot = get_bot()
            await bot.call_api(
                "set_msg_emoji_like",
                message_id=int(message_id),
                emoji_id="212",
                set="true" if status else "false",
            )
        except Exception as e:
            logger.error(f"设置消息emoji失败: {e} | 如果协议端不支持该功能，请关闭配置 `SESSION_PROCESSING_WITH_EMOJI`")
            return False
        else:
            return True



from .base import PromptTemplate, register_template


class BasePracticePrompt_question(PromptTemplate):
    """对话示例问题基类"""
    one_time_code: str


class BasePracticePrompt_response(PromptTemplate):
    """对话示例回答基类"""
    one_time_code: str
    enable_cot: bool
    enable_at: bool


@register_template("practice.j2", "practice_question_1")
class PracticePrompt_question_1(BasePracticePrompt_question):
    pass


@register_template("practice.j2", "practice_response_1")
class PracticePrompt_response_1(BasePracticePrompt_response):
    pass


@register_template("practice.j2", "practice_question_2")
class PracticePrompt_question_2(BasePracticePrompt_question):
    pass


@register_template("practice.j2", "practice_response_2")
class PracticePrompt_response_2(BasePracticePrompt_response):
    pass



"""
SSE 路由模块
==========

负责注册和挂载SSE协议的FastAPI路由。
"""

import inspect
import uuid
from typing import Any, Dict, Optional, Union

from fastapi import APIRouter, Body, Header, HTTPException, Request
from sse_starlette.sse import EventSourceResponse

from nekro_agent.adapters.utils import adapter_utils
from nekro_agent.core import logger

from .commands import (
    _command_handlers,
    _command_models,
    set_client_manager,
)
from .core.client import SseClient, SseClientManager, sse_stream

# 全局客户端管理器 - 将由 mount_adapter_router 初始化
client_manager: Optional[SseClientManager] = None


# 设置client_manager的函数，在路由挂载时调用
def set_router_client_manager(manager: SseClientManager) -> None:
    """设置路由使用的客户端管理器

    Args:
        manager: 客户端管理器实例
    """
    global client_manager
    client_manager = manager
    # 同时也设置commands.py中的client_manager
    set_client_manager(manager)


def create_sse_response(request: Request, client: Union[SseClient, str]) -> EventSourceResponse:
    """创建SSE响应

    Args:
        request: 请求对象
        client: 客户端实例或客户端名称

    Returns:
        EventSourceResponse: SSE响应对象
    """
    assert client_manager is not None, "客户端管理器未初始化"
    # 如果是字符串，则创建新客户端
    if isinstance(client, str):
        existing_client = client_manager.get_client_by_name(client)
        client = existing_client or client_manager.register_client(client)

    # 返回SSE响应
    return EventSourceResponse(sse_stream(request, client))


router = APIRouter()


# 辅助函数，用于抽象化异常抛出
def _raise_http_exception(status_code: int, detail: str) -> None:
    """抛出HTTP异常

    Args:
        status_code: HTTP状态码
        detail: 异常详情
    """
    raise HTTPException(status_code=status_code, detail=detail)


@router.post("/connect")
async def command_endpoint(
    command_payload: Dict[str, Any] = Body(...),
    client_id_header: Optional[str] = Header(None, alias="X-Client-ID"),
    access_key_header: Optional[str] = Header(None, alias="X-Access-Key"),
):
    """命令处理端点"""
    # 访问密钥校验
    sse_adapter = adapter_utils.get_adapter("sse")
    if sse_adapter.config.ACCESS_KEY and access_key_header != sse_adapter.config.ACCESS_KEY:
        _raise_http_exception(401, "无效的访问密钥")

    cmd = command_payload.get("cmd")
    if not cmd:
        raise HTTPException(status_code=400, detail="缺少cmd字段")

    logger.info(f"收到命令: {cmd} (Client-ID: {client_id_header})")

    handler = _command_handlers.get(cmd)
    if not handler:
        raise HTTPException(status_code=400, detail=f"未知命令: {cmd}")

    try:
        model_cls = _command_models.get(cmd)
        # 使用 pydantic 模型验证和转换数据
        command_data_obj = model_cls(**command_payload) if model_cls else command_payload

        sig = inspect.signature(handler)
        params_for_handler: Dict[str, Any] = {}

        if "command" in sig.parameters:
            params_for_handler["command"] = command_data_obj

        current_cmd_client_id = client_id_header  # 使用从header获取的ID

        # 仅当处理器需要 client_id 时才传递
        if "client_id" in sig.parameters:
            # 对于非注册命令，如果处理器需要 client_id 但未提供，则报错
            if not current_cmd_client_id and cmd != "register":
                _raise_http_exception(400, f"命令 '{cmd}' 需要 X-Client-ID 请求头")
            params_for_handler["client_id"] = current_cmd_client_id

        # 更新心跳 (使用已设置为 adapter.client_manager 的模块级 client_manager)
        if current_cmd_client_id and client_manager:
            client_obj = client_manager.get_client(current_cmd_client_id)
            if client_obj:
                client_obj.update_heartbeat()
            elif cmd != "register":  # 如果不是注册，但客户端不存在，可能记录一个警告
                logger.warning(f"Command '{cmd}' from unknown client_id: {current_cmd_client_id}")

        # 确保客户端管理器已初始化
        if client_manager is None and cmd == "register":
            _raise_http_exception(500, "SSE服务未正确初始化，客户端管理器未设置")

        return await handler(**params_for_handler)

    except HTTPException:
        raise
    except Exception as e:
        logger.exception(f"处理命令失败: {cmd}, 错误: {e!r}")
        raise HTTPException(status_code=500, detail=f"处理 {cmd} 命令失败: {e!s}") from e


@router.get("/connect")
async def sse_endpoint(
    request: Request,
    client_name: str = "",
    client_id_query: Optional[str] = None,  # 名称区分来自查询参数
    platform: str = "unknown",
    access_key: Optional[str] = None,
):
    """SSE 连接端点"""
    # 访问密钥校验
    sse_adapter = adapter_utils.get_adapter("sse")
    if sse_adapter.config.ACCESS_KEY and access_key != sse_adapter.config.ACCESS_KEY:
        logger.error("SSE 连接失败: 无效的访问密钥。")
        raise HTTPException(status_code=401, detail="无效的访问密钥")

    if not client_manager:
        logger.error("SSE 连接失败: client_manager 未初始化。请检查 SSEAdapter 初始化流程。")
        raise HTTPException(status_code=500, detail="SSE 服务内部错误，未能正确初始化。")

    active_client: SseClient
    effective_client_id = client_id_query

    if effective_client_id:
        client_from_manager = client_manager.get_client(effective_client_id)
        if client_from_manager:
            active_client = client_from_manager
            # 只更新心跳，不更新平台信息
            active_client.update_heartbeat()
            logger.info(f"SSE客户端重连: ID={effective_client_id}, Name='{client_from_manager.name}', Platform='{platform}'")
        else:
            # 如果提供了ID但找不到，则用此ID注册新客户端
            # client_name 优先使用查询参数，其次用ID做名称
            name_for_new_client = client_name or effective_client_id
            active_client = client_manager.register_client(name_for_new_client, platform)
            logger.info(
                f"SSE新客户端注册 (通过ID): ID={effective_client_id}, Name='{name_for_new_client}', Platform='{platform}'",
            )
    else:
        # 没有提供client_id，基于client_name注册新客户端
        if not client_name:  # 如果连name都没有，就生成一个
            client_name = f"sse-client-{uuid.uuid4().hex[:8]}"
            logger.info(f"SSE客户端名称未提供，已自动生成: {client_name}")
        active_client = client_manager.register_client(client_name, platform)
        logger.info(f"SSE新客户端注册: Name='{client_name}', Platform='{platform}', Assigned ID='{active_client.client_id}'")

    # 更新心跳，确保客户端活跃
    active_client.update_heartbeat()
    return EventSourceResponse(sse_stream(request, active_client))


logger.info("SSE Router mounted on adapter.router and using adapter.client_manager!")



from abc import ABC, abstractmethod
from pathlib import Path
from typing import Dict, Generic, List, Optional, Tuple, Type, TypeVar, cast

from fastapi import APIRouter
from jinja2 import Environment
from nonebot import logger
from pydantic import BaseModel, Field

from nekro_agent.core.core_utils import ConfigBase
from nekro_agent.core.os_env import OsEnv
from nekro_agent.schemas.chat_message import ChatType
from nekro_agent.services.agent.templates.base import PromptTemplate, register_template

from .schemas.platform import (
    PlatformChannel,
    PlatformSendRequest,
    PlatformSendResponse,
    PlatformUser,
)


class AdapterMetadata(BaseModel):
    """适配器元数据"""

    name: str
    description: str
    version: str = "1.0.0"
    author: str = ""
    homepage: str = ""
    tags: List[str] = []


class BaseAdapterConfig(ConfigBase):
    """适配器配置基类"""

    SESSION_ENABLE_AT: bool = Field(
        default=True,
        title="启用 @用户 功能",
        description="关闭后 AI 发送的 @用户 消息将被解析为纯文本用户名，避免反复打扰用户",
    )
    SESSION_PROCESSING_WITH_EMOJI: bool = Field(
        default=True,
        title="显示处理中表情反馈",
        description="当 AI 开始处理消息时，对应消息会显示处理中表情反馈",
    )


# 定义配置类型变量，约束为 BaseAdapterConfig 子类
TConfig = TypeVar("TConfig", bound=BaseAdapterConfig)
T = TypeVar("T", bound="BaseAdapter")


class BaseAdapter(Generic[TConfig], ABC):
    """适配器基类"""

    _router: APIRouter  # 实例变量的类型注解
    _Configs: Type[TConfig]
    _config: TConfig

    def __init__(self, config_cls: Type[TConfig] = BaseAdapterConfig):
        self._Configs = config_cls
        self._adapter_config_path = Path(OsEnv.DATA_DIR) / "configs" / self.key / "config.yaml"
        self._config = self.get_config(self._Configs)

        # 注册配置到统一配置系统
        from nekro_agent.core.core_utils import ConfigManager

        ConfigManager.register_config(f"adapter_{self.key}", self._config)

    @property
    @abstractmethod
    def key(self) -> str:
        """适配器唯一标识"""
        raise NotImplementedError

    @property
    @abstractmethod
    def metadata(self) -> AdapterMetadata:
        """适配器元数据"""
        raise NotImplementedError

    def get_adapter_router(self) -> APIRouter:
        """获取适配器路由"""
        return APIRouter()

    @property
    def router(self) -> APIRouter:
        """获取适配器路由"""
        if hasattr(self, "_router"):
            return self._router
        self._router = self.get_adapter_router()
        return self._router

    def get_config(self, config_cls: Type[TConfig]) -> TConfig:
        """获取适配器配置"""
        if not hasattr(self, "_config"):
            self._config = self._Configs.load_config(file_path=self._adapter_config_path)
            self._config.dump_config(self._adapter_config_path)
        return cast(config_cls, self._config)

    @property
    def config(self) -> TConfig:
        """获取适配器配置"""
        return self.get_config(self._Configs)

    @property
    def config_path(self) -> Path:
        """获取适配器配置路径"""
        return self._adapter_config_path

    def cast(self, adapter_type: Type[T]) -> T:
        """转换适配器类型"""
        return cast(adapter_type, self)

    @property
    def chat_key_rules(self) -> List[str]:
        return [
            "Group chat: `platform-group_123456` (where 123456 is the group number)",
            "Private chat: `platform-private_123456` (where 123456 is the user's QQ number)",
        ]

    def get_docs_path(self) -> Path:
        """获取适配器文档路径"""
        return Path(__file__).parent.parent / self.key / "README.md"

    @abstractmethod
    async def init(self) -> None:
        """初始化适配器"""
        raise NotImplementedError

    @abstractmethod
    async def cleanup(self) -> None:
        """清理适配器"""
        raise NotImplementedError

    async def set_dialog_example(self) -> Optional[List[PromptTemplate]]:
        """自定义对话示例"""
        return None

    async def get_jinja_env(self) -> Optional[Environment]:
        """返回jinja模板"""
        return None

    @abstractmethod
    async def forward_message(self, request: PlatformSendRequest) -> PlatformSendResponse:
        """推送消息到协议端

        Args:
            request: 协议端发送请求，包含已经预处理好的消息数据

        Returns:
            PlatformSendResponse: 发送结果
        """
        raise NotImplementedError

    @abstractmethod
    async def get_self_info(self) -> PlatformUser:
        """获取自身信息"""
        raise NotImplementedError

    @abstractmethod
    async def get_user_info(self, user_id: str, channel_id: str) -> PlatformUser:
        """获取用户(或者群聊用户)信息"""
        raise NotImplementedError

    @abstractmethod
    async def get_channel_info(self, channel_id: str) -> PlatformChannel:
        """获取频道信息"""
        raise NotImplementedError

    async def set_message_reaction(self, message_id: str, status: bool = True) -> bool:  # noqa: ARG002
        """设置消息反应（可选实现）

        Args:
            message_id (str): 消息ID
            status (bool): True为设置反应，False为取消反应

        Returns:
            bool: 是否成功设置
        """
        # 默认实现：不支持消息反应功能
        return False

    # region 辅助方法

    def build_chat_key(self, channel_id: str) -> str:
        """构建聊天标识"""
        return f"{self.key}-{channel_id}"

    def parse_chat_key(self, chat_key: str) -> Tuple[str, str]:
        """解析聊天标识

        Args:
            chat_key: 聊天标识

        Returns:
            Tuple[str, str]: (adapter_key, channel_id)
        """
        parts = chat_key.split("-")

        if len(parts) != 2:
            raise ValueError(f"无效的聊天标识: {chat_key}")

        adapter_key = parts[0]
        channel_id = parts[1]

        return adapter_key, channel_id

    def parse_channel_id(self, chat_key: str) -> str:
        """解析聊天标识中的频道ID"""
        return self.parse_chat_key(chat_key)[1]

    # endregion



import time
from datetime import datetime
from typing import Any, Dict, Optional, Type

from nonebot import on_notice
from nonebot.adapters.minecraft import Bot, NoticeEvent
from nonebot.matcher import Matcher

from nekro_agent.core.config import config
from nekro_agent.core.logger import logger
from nekro_agent.models.db_chat_channel import DBChatChannel
from nekro_agent.models.db_user import DBUser
from nekro_agent.schemas.chat_message import ChatMessage, ChatType
from nekro_agent.services.message_service import message_service
from nekro_agent.services.notice_service import (
    BaseNoticeHandler,
    NoticeConfig,
    NoticeResult,
)
from nekro_agent.tools.time_util import format_duration


class PlayerJoinNoticeHandler(BaseNoticeHandler):
    """玩家加入通知处理器"""

    def get_notice_config(self) -> NoticeConfig:
        return NoticeConfig(force_tome=True, use_system_sender=True)

    def match(self, _db_chat_channel: DBChatChannel, event_dict: Dict[str, Any]) -> Optional[Dict[str, str]]:
        if event_dict["sub_type"] != "join":
            return None
        return {
            "user_id": str(event_dict["player"].nickname),
        }

    async def format_message(self, _db_chat_channel: DBChatChannel, info: Dict[str, str]) -> str:
        return f"(玩家 {info['user_id']} 加入了服务器)"

class PlayerQuitNoticeHandler(BaseNoticeHandler):
    """玩家离开通知处理器"""

    def get_notice_config(self) -> NoticeConfig:
        return NoticeConfig(force_tome=True, use_system_sender=True)

    def match(self, _db_chat_channel: DBChatChannel, event_dict: Dict[str, Any]) -> Optional[Dict[str, str]]:
        if event_dict["sub_type"] != "quit":
            return None
        return {
            "user_id": str(event_dict["player"].nickname),
        }

    async def format_message(self, _db_chat_channel: DBChatChannel, info: Dict[str, str]) -> str:
        return f"(玩家 {info['user_id']} 离开了服务器)"

class NoticeHandlerManager:
    """通知处理器管理器"""

    def __init__(self):
        self._handlers: list[BaseNoticeHandler] = []

    def register(self, handler: BaseNoticeHandler):
        """注册处理器"""
        self._handlers.append(handler)

    async def handle(self, event: NoticeEvent, _bot: Bot, db_chat_channel: DBChatChannel) -> Optional[NoticeResult]:
        """处理通知事件

        Args:
            event (NoticeEvent): 通知事件
            _bot (Bot): 机器人实例

        Returns:
            Optional[NoticeResult]: 处理结果，包含处理器实例和通知信息
        """
        # 预处理事件为字典
        event_dict = dict(event)

        for handler in self._handlers:
            if info := handler.match(db_chat_channel, event_dict):
                return NoticeResult(
                    handler=handler,
                    info=info,
                )
        return None

    def get_sender_platform_userid(self, info: Dict[str, str]) -> str:
        """获取发送者平台用户ID"""
        return info["user_id"]

notice_manager = NoticeHandlerManager()

notice_manager.register(PlayerJoinNoticeHandler())
notice_manager.register(PlayerQuitNoticeHandler())

"""通用通知匹配器"""
notice_matcher: Type[Matcher] = on_notice(priority=99999, block=False)


@notice_matcher.handle()
async def _(_: Matcher, event: NoticeEvent, bot: Bot):
    # 处理通知事件
    chat_key = "minecraft-" + event.server_name
    db_chat_channel: DBChatChannel = await DBChatChannel.get_channel(chat_key=chat_key)
    result = await notice_manager.handle(event, bot, db_chat_channel)
    if not result:
        event_dict = dict(event)
        logger.debug(
            f"收到未处理的通知类型: {event_dict}\n"
            f"notice_type: {event_dict.get('notice_type')}\n"
            f"sub_type: {event_dict.get('sub_type')}\n",
        )
        return

    handler = result["handler"]
    info = result["info"]

    # 格式化消息
    content_text = await handler.format_message(db_chat_channel, info)

    if handler.config.use_system_sender:
        # 使用系统消息
        await message_service.push_system_message(
            chat_key=chat_key,
            agent_messages=content_text,
            trigger_agent=handler.config.force_tome,
        )
    else:
        # 使用普通消息
        platform_userid: str = handler.get_sender_platform_userid(info)
        user: Optional[DBUser] = await DBUser.get_or_none(
            adapter_key=db_chat_channel.adapter_key,
            platform_userid=platform_userid,
        )
        sender_nickname = event.player.nickname

        if user and not user.is_active:
            logger.info(f"用户 {platform_userid} 被封禁，封禁结束时间: {user.ban_until}")
            return

        chat_message: ChatMessage = ChatMessage(
            message_id="",
            sender_id=str(user.id) if user else platform_userid,
            sender_name=user.username if user else sender_nickname,
            sender_nickname=sender_nickname,
            adapter_key=db_chat_channel.adapter_key,
            platform_userid=platform_userid,
            is_tome=1 if handler.config.force_tome else 0,
            is_recalled=False,
            chat_key=chat_key,
            chat_type=ChatType.GROUP,
            content_text=content_text,
            content_data=[],
            raw_cq_code="",
            ext_data={},
            send_timestamp=int(time.time()),
        )
        await message_service.push_human_message(message=chat_message, user=user)


from enum import Enum
from time import time
from typing import TYPE_CHECKING, List, Optional, Union

from pydantic import BaseModel, Field

from nekro_agent.models.db_chat_channel import DBChatChannel, DefaultPreset
from nekro_agent.models.db_preset import DBPreset
from nekro_agent.schemas.chat_message import ChatMessageSegment, ChatType

from .extra import PlatformMessageExt

if TYPE_CHECKING:
    from nekro_agent.adapters.interface.base import BaseAdapter


class PlatformUser(BaseModel):
    platform_name: str = Field(..., description="平台名称")
    user_id: str = Field(..., description="用户ID")
    user_name: str = Field(..., description="用户名称")
    user_avatar: str = Field(default="", description="用户头像")


class PlatformChannel(BaseModel):
    channel_id: str = Field(..., description="频道ID")
    channel_name: str = Field(..., description="频道名称")
    channel_type: ChatType = Field(default=ChatType.UNKNOWN, description="频道类型")
    channel_avatar: str = Field(default="", description="频道头像")

    async def get_db_chat_channel(self, adapter: "BaseAdapter") -> DBChatChannel:
        return await DBChatChannel.get_or_create(
            adapter_key=adapter.key,
            channel_id=self.channel_id,
            channel_type=self.channel_type,
        )

    async def get_preset(self, adapter: "BaseAdapter") -> Union[DBPreset, DefaultPreset]:
        return await (await self.get_db_chat_channel(adapter)).get_preset()


class PlatformMessage(BaseModel):
    message_id: str = Field(..., description="消息ID")
    sender_id: str = Field(..., description="发送者ID")
    sender_name: str = Field(..., description="发送者名称")
    sender_nickname: str = Field(default="", description="发送者昵称")
    sender_avatar: str = Field(default="", description="发送者头像")
    content_data: List[ChatMessageSegment] = Field(default=[], description="消息内容")
    content_text: str = Field(default="", description="消息内容")
    is_tome: bool = Field(default=False, description="是否为 @ 消息")
    timestamp: int = Field(default_factory=lambda: int(time()), description="消息时间戳")
    is_self: bool = Field(default=False, description="是否为自己发送的消息")
    ext_data: Optional[PlatformMessageExt] = Field(default=PlatformMessageExt(), description="扩展数据")


# ========================================================================================
# |                          协议端消息发送相关数据结构                                      |
# ========================================================================================


class PlatformSendSegmentType(str, Enum):
    """协议端消息段类型"""

    TEXT = "text"
    AT = "at"
    IMAGE = "image"  # 以图片/富文本形式发送
    FILE = "file"  # 以文件上传形式发送


class PlatformAtSegment(BaseModel):
    """@ 消息段"""

    platform_user_id: str = Field(..., description="平台用户ID")
    nickname: Optional[str] = Field(default=None, description="用户昵称")


class PlatformSendSegment(BaseModel):
    """协议端发送消息段

    这是主程序向协议端传递的标准化消息段结构，
    协议端根据 type 字段决定具体的发送方式
    """

    type: PlatformSendSegmentType = Field(..., description="消息段类型")
    content: str = Field(default="", description="消息内容")
    at_info: Optional[PlatformAtSegment] = Field(default=None, description="@ 信息（仅当 type=AT 时有效）")
    file_path: Optional[str] = Field(
        default=None,
        description="文件路径（仅当 type=IMAGE/FILE 时有效，已转换为协议端可访问的路径）",
    )


class PlatformSendRequest(BaseModel):
    """协议端发送请求

    协议端根据 segments 中每个消息段的 type 来决定具体的发送方式：
    - FILE: 文件上传模式
    - IMAGE: 图片/富文本模式
    - TEXT/AT: 普通文本消息
    """

    chat_key: str = Field(..., description="会话标识")
    segments: List[PlatformSendSegment] = Field(default=[], description="消息段列表")
    ref_msg_id: Optional[str] = Field(default=None, description="引用消息ID")


class PlatformSendResponse(BaseModel):
    """协议端发送响应"""

    success: bool = Field(..., description="是否发送成功")
    error_message: Optional[str] = Field(default=None, description="错误信息")
    message_id: Optional[str] = Field(default=None, description="消息ID")



import shutil
from pathlib import Path
from typing import Type, Union

from nekro_agent.core.os_env import SANDBOX_SHARED_HOST_DIR, USER_UPLOAD_DIR
from nekro_agent.tools.common_util import (
    download_file,
    download_file_from_base64,
    download_file_from_bytes,
)

from .path_convertor import (
    convert_filepath_to_sandbox_shared_path,
    convert_filepath_to_sandbox_upload_path,
    convert_to_host_path,
    get_upload_file_path,
)


class FileUtils:

    @classmethod
    def sandbox2host(cls, sandbox_path: str | Path, chat_key: str, container_key: str) -> Path:
        """将沙盒内互通路径转换为宿主机路径

        用来转换 AI 提供的沙盒内互通路径为插件可访问的宿主机路径

        Args:
            sandbox_path: 沙盒内互通路径
            chat_key: 聊天会话ID
            container_key: 容器ID

        Returns:
            Path: 宿主机路径
        """
        return convert_to_host_path(Path(sandbox_path), chat_key, container_key)

    @classmethod
    def host_shared2sandbox(cls, host_filepath: str | Path) -> Path:
        """将宿主机沙盒共享目录文件路径转换为沙盒内路径

        Args:
            host_filepath: 宿主机上传路径

        Returns:
            Path: 沙盒内路径
        """
        return convert_filepath_to_sandbox_shared_path(Path(host_filepath))

    @classmethod
    def host_upload2sandbox(cls, host_filepath: str | Path) -> Path:
        """将宿主机上传目录文件路径转换为沙盒内路径"""
        return convert_filepath_to_sandbox_upload_path(Path(host_filepath))

    @classmethod
    def get_sandbox_shared_host_path(cls, container_key: str) -> Path:
        """获取沙盒共享目录的宿主机路径

        Args:
            chat_key: 聊天会话ID
            container_key: 容器ID
        """
        path = cls.to_absolute_path(SANDBOX_SHARED_HOST_DIR) / container_key
        path.mkdir(parents=True, exist_ok=True)
        return path

    @classmethod
    def get_upload_file_path(cls, from_chat_key: str, file_name: str = "", use_suffix: str = "", seed: str = "") -> str:
        """获取或生成一个可用的上传文件路径

        Args:
            from_chat_key: 聊天会话ID
            file_name: 文件名
            use_suffix: 指定文件后缀
            seed: 种子
        """
        return get_upload_file_path(from_chat_key, file_name, use_suffix, seed)

    @classmethod
    def to_absolute_path(cls, path: str | Path) -> Path:
        """将相对路径转换为绝对路径

        Args:
            path: 路径
        """
        return Path(path) if str(path).startswith("/") else Path(path).resolve()


class FileSystem:
    """文件系统工具类，提供插件应用与沙盒内文件系统互通的工具"""

    def __init__(self, chat_key: str, container_key: str):
        self._chat_key = chat_key
        self._container_key = container_key

    @property
    def chat_key(self) -> str:
        return self._chat_key

    @property
    def container_key(self) -> str:
        return self._container_key

    @property
    def file_utils(self) -> Type[FileUtils]:
        return FileUtils

    @property
    def shared_path(self) -> Path:
        """获取当前会话的文件共享目录"""
        return FileUtils.get_sandbox_shared_host_path(self.container_key)

    @property
    def upload_path(self) -> Path:
        """获取当前会话的文件上传目录"""
        return Path(USER_UPLOAD_DIR) / self.chat_key

    def get_file(self, file_path: Path | str) -> Path:
        """从 AI 提供的沙盒内互通路径获取可访问的文件对象

        Args:
            file_path: 文件路径

        Returns:
            Path: 文件对象
        """
        path = Path(file_path)
        if path.is_relative_to(Path("/app/shared")):
            return self.shared_path / path.relative_to(Path("/app/shared"))
        if path.is_relative_to(Path("/app/uploads")):
            return self.upload_path / path.relative_to(Path("/app/uploads"))
        raise ValueError(f'文件 "{path}" 不在合法的沙盒内容共享目录或上传目录下，无法映射到应用路径')

    def forward_file(self, file_path: Path | str) -> str:
        """通过文件路径向 AI 提供一个可访问的沙盒内互通文件路径

        注意: 需要先把文件放到 shared_path 或 upload_path 下

        Args:
            file_path: 文件路径

        Returns:
            str: 可访问的文件路径
        """
        path = Path(file_path)
        # 检查文件是否在共享目录下
        if path.is_relative_to(self.shared_path):
            rel_path = path.relative_to(self.shared_path)
            return str(self.file_utils.host_shared2sandbox(rel_path))
        if path.is_relative_to(self.upload_path):
            rel_path = path.relative_to(self.upload_path)
            return str(self.file_utils.host_upload2sandbox(rel_path))
        raise ValueError(f'文件 "{path}" 不在合法的应用共享目录或上传目录下，无法映射到沙盒路径')

    async def mixed_forward_file(self, file: Union[str, bytes, Path], file_name: str = "") -> str:
        """通过 url/base64/bytes/path 向 AI 提供一个可访问的沙盒内互通文件路径

        Args:
            file: 文件 URL/base64/bytes/path

        Returns:
            str: 可访问的文件路径
        """
        if isinstance(file, str):
            if file.startswith(("http", "https")):
                file_path, file_name = await download_file(file, from_chat_key=self.chat_key, file_name=file_name)
            elif file.startswith("data:"):
                file_path, file_name = await download_file_from_base64(file, from_chat_key=self.chat_key, file_name=file_name)
        elif isinstance(file, bytes):
            file_path, file_name = await download_file_from_bytes(file, from_chat_key=self.chat_key, file_name=file_name)
        elif isinstance(file, Path):
            if file.is_relative_to(self.shared_path):
                file_path = self.shared_path / file.relative_to(self.shared_path)
            elif file.is_relative_to(self.upload_path):
                file_path = self.upload_path / file.relative_to(self.upload_path)
            else:
                shutil.copy(file, self.shared_path / file.name)
                file_path = self.shared_path / file.name
        else:
            raise TypeError(f"不支持的文件类型: {type(file)}")
        return self.forward_file(file_path)



"""
SSE 消息转换器
===========

负责在平台消息格式和SSE消息格式之间进行转换。

主要功能:
1. 平台消息段 -> SSE消息段的转换
2. SSE消息段 -> 平台消息段的转换
3. 消息内容解析与构建
"""

import time
from pathlib import Path
from typing import List

from nekro_agent.adapters.interface.schemas.platform import (
    PlatformMessage,
    PlatformSendSegment,
    PlatformSendSegmentType,
)
from nekro_agent.adapters.sse.sdk.models import (
    AtSegment,
    FileSegment,
    ImageSegment,
    MessageSegmentType,
    ReceiveMessage,
    SendMessage,
    TextSegment,
    at,
    file,
    image,
    text,
)
from nekro_agent.adapters.sse.tools.common import get_file_base64, get_file_info
from nekro_agent.adapters.utils import adapter_utils
from nekro_agent.core import logger
from nekro_agent.schemas.chat_message import (
    ChatMessageSegment,
    ChatMessageSegmentAt,
    ChatMessageSegmentFile,
    ChatMessageSegmentImage,
    ChatMessageSegmentType,
)


class SseMessageConverter:
    """SSE 消息转换器

    用于在不同消息格式之间进行转换
    """

    @staticmethod
    async def platform_to_sse_message(channel_id: str, segments: List[PlatformSendSegment]) -> SendMessage:
        """将平台消息段转换为SSE消息

        Args:
            channel_id: 频道ID
            segments: 平台消息段列表

        Returns:
            SendMessage: SSE消息
        """
        sse_segments = []

        for segment in segments:
            if segment.type == PlatformSendSegmentType.TEXT:
                # 文本消息
                sse_segments.append(text(segment.content))

            elif segment.type == PlatformSendSegmentType.IMAGE:
                # 图片消息
                if segment.file_path:
                    try:
                        # 图片文件路径处理
                        file_name = Path(segment.file_path).name
                        file_suffix = Path(segment.file_path).suffix
                        # 获取base64编码和MIME类型
                        base64_url, mime_type, _ = await get_file_base64(segment.file_path)

                        sse_segments.append(
                            image(base64_url=base64_url, name=file_name, mime_type=mime_type, suffix=file_suffix),
                        )
                    except Exception as e:
                        logger.error(f"图片处理失败: {e}")
                        sse_segments.append(text(f"[图片上传失败: {Path(segment.file_path).name}]"))

            elif segment.type == PlatformSendSegmentType.FILE:
                # 文件消息
                if segment.file_path:
                    try:
                        # 文件路径处理
                        file_name = Path(segment.file_path).name
                        file_suffix = Path(segment.file_path).suffix
                        # 获取base64编码和MIME类型
                        base64_url, mime_type, _ = await get_file_base64(segment.file_path)

                        # 获取文件大小
                        file_bytes, _, _ = await get_file_info(segment.file_path)
                        file_size = len(file_bytes)

                        sse_segments.append(
                            file(
                                base64_url=base64_url,
                                name=file_name,
                                size=file_size,
                                mime_type=mime_type,
                                suffix=file_suffix,
                            ),
                        )
                    except Exception as e:
                        logger.error(f"文件处理失败: {e}")
                        sse_segments.append(text(f"[文件上传失败: {Path(segment.file_path).name}]"))

            elif segment.type == PlatformSendSegmentType.AT and segment.at_info:
                # @消息
                sse_segments.append(
                    at(
                        user_id=segment.at_info.platform_user_id,
                        nickname=segment.at_info.nickname,
                    ),
                )

        return SendMessage(
            channel_id=channel_id,
            segments=sse_segments,
            timestamp=int(time.time()),
        )

    @staticmethod
    async def sse_to_platform_message(message: ReceiveMessage) -> PlatformMessage:
        """将SSE接收消息转换为平台消息

        Args:
            message: SSE接收消息

        Returns:
            PlatformMessage: 平台消息
        """
        adapter = adapter_utils.get_adapter("sse")
        # 消息段列表
        content_data: List[ChatMessageSegment] = []
        # 文本内容
        content_text = ""

        # 构建chat_key用于文件处理
        chat_key = adapter.build_chat_key(message.channel_id)

        # 遍历消息段
        for segment in message.segments:
            if segment.type == MessageSegmentType.TEXT:
                # 必须先判断类型，然后才能安全访问属性
                if isinstance(segment, TextSegment):
                    content_text += segment.content
                    content_data.append(
                        ChatMessageSegment(
                            type=ChatMessageSegmentType.TEXT,
                            text=segment.content,
                        ),
                    )

            elif segment.type == MessageSegmentType.IMAGE:
                # 图片消息段
                if isinstance(segment, ImageSegment):
                    if segment.base64_url:
                        # 处理文件名
                        file_name = segment.name or ""
                        # 使用现成的base64方法处理
                        image_msg = await ChatMessageSegmentImage.create_from_base64(
                            base64_str=segment.base64_url,
                            from_chat_key=chat_key,
                            file_name=file_name,
                        )
                        content_data.append(image_msg)

                    elif segment.url:
                        # 处理文件名
                        file_name = segment.name or ""
                        # 使用现成的URL方法处理
                        image_msg = await ChatMessageSegmentImage.create_from_url(
                            url=segment.url,
                            from_chat_key=chat_key,
                            file_name=file_name,
                        )
                        content_data.append(image_msg)

            elif segment.type == MessageSegmentType.FILE:
                # 文件消息段
                if isinstance(segment, FileSegment):
                    if segment.base64_url:
                        # 处理文件名
                        file_name = segment.name or ""
                        # 使用现成的base64方法处理
                        file_msg = await ChatMessageSegmentFile.create_from_base64(
                            base64_str=segment.base64_url,
                            from_chat_key=chat_key,
                            file_name=file_name,
                        )
                        content_data.append(file_msg)

                    elif segment.url:
                        # 处理文件名
                        file_name = segment.name or ""
                        # 使用现成的URL方法处理
                        file_msg = await ChatMessageSegmentFile.create_from_url(
                            url=segment.url,
                            from_chat_key=chat_key,
                            file_name=file_name,
                        )
                        content_data.append(file_msg)

            elif segment.type == MessageSegmentType.AT:
                # @消息段
                if isinstance(segment, AtSegment):
                    nickname = segment.nickname or segment.user_id
                    content_data.append(
                        ChatMessageSegmentAt(
                            type=ChatMessageSegmentType.AT,
                            text=f"@{nickname}",
                            target_platform_userid=segment.user_id,
                            target_nickname=nickname,
                        ),
                    )
                    # 添加到文本内容
                    content_text += f"@{nickname} "
            else:
                logger.warning(f"未知的消息段类型: {segment.type}")

        return PlatformMessage(
            message_id=message.msg_id,
            sender_id=message.from_id,
            sender_name=message.from_name,
            sender_nickname=message.from_nickname or message.from_name,
            content_data=content_data,
            content_text=content_text,
            is_tome=message.is_to_me,
            timestamp=message.timestamp,
            is_self=message.is_self,
        )



"""配置服务

提供通用的配置管理功能，可以应用于系统配置和插件配置。
"""

import inspect
import json
from enum import Enum
from pathlib import Path
from typing import (
    Any,
    Dict,
    List,
    Literal,
    Optional,
    Tuple,
    TypeVar,
    get_args,
    get_origin,
)

from pydantic import BaseModel

from nekro_agent.core import logger
from nekro_agent.core.config import CHANNEL_CONFIG_DIR
from nekro_agent.core.core_utils import ConfigBase, ConfigManager
from nekro_agent.core.os_env import OsEnv

T = TypeVar("T", bound=ConfigBase)


class ConfigType(str, Enum):
    """配置类型"""

    SYSTEM = "system"
    PLUGIN = "plugin"
    MODEL_GROUPS = "model_groups"
    ADAPTER = "adapter"
    SESSION = "session"


def _analyze_field_type(field_type: Any, value: Any = None) -> Dict[str, Any]:
    """分析字段类型

    Args:
        field_type: 字段类型注解
        value: 字段值

    Returns:
        Dict[str, Any]: 包含类型分析结果的字典
    """
    origin = get_origin(field_type)
    args = get_args(field_type)

    result = {
        "base_type": "str",
        "is_complex": False,
        "element_type": None,
        "key_type": None,
        "value_type": None,
        "field_schema": None,  # 新增：字段模式信息
    }

    # 检查是否是Pydantic模型
    if inspect.isclass(field_type) and issubclass(field_type, BaseModel):
        result.update(
            {
                "base_type": "object",
                "is_complex": True,
                "field_schema": _get_model_schema(field_type),
            },
        )
        return result

    # 处理泛型类型
    if origin is list:
        result["base_type"] = "list"
        if args and len(args) > 0:
            element_type = args[0]
            if inspect.isclass(element_type) and issubclass(element_type, BaseModel):
                # List[CustomModel]
                result.update(
                    {
                        "is_complex": True,
                        "element_type": "object",
                        "field_schema": _get_model_schema(element_type),
                    },
                )
            else:
                # List[简单类型]
                result.update(
                    {
                        "element_type": _get_simple_type_name(element_type),
                    },
                )
        else:
            # 无类型参数的list，根据值推断
            element_type = _get_field_type(value[0]) if value and len(value) > 0 else "str"
            result.update(
                {
                    "element_type": element_type,
                },
            )

    elif origin is dict:
        result["base_type"] = "dict"
        if len(args) >= 2:
            key_type, value_type = args[0], args[1]
            if inspect.isclass(value_type) and issubclass(value_type, BaseModel):
                # Dict[str, CustomModel]
                result.update(
                    {
                        "is_complex": True,
                        "key_type": _get_simple_type_name(key_type),
                        "value_type": "object",
                        "field_schema": _get_model_schema(value_type),
                    },
                )
            else:
                # Dict[str, 简单类型]
                result.update(
                    {
                        "key_type": _get_simple_type_name(key_type),
                        "value_type": _get_simple_type_name(value_type),
                    },
                )
        else:
            # 无类型参数的dict
            result.update(
                {
                    "key_type": "str",
                    "value_type": "str",
                },
            )

    else:
        # 简单类型
        result.update(
            {
                "base_type": _get_field_type(value) if value is not None else _get_simple_type_name(field_type),
            },
        )

    return result


def _get_simple_type_name(type_annotation: Any) -> str:
    """获取简单类型的名称"""
    if type_annotation is str:
        return "str"
    if type_annotation is int:
        return "int"
    if type_annotation is float:
        return "float"
    if type_annotation is bool:
        return "bool"
    return "str"


def _get_model_schema(model_class: Any) -> Dict[str, Any]:
    """获取Pydantic模型的字段模式信息

    Args:
        model_class: Pydantic模型类

    Returns:
        Dict[str, Any]: 字段模式信息
    """
    if not inspect.isclass(model_class) or not issubclass(model_class, BaseModel):
        return {}

    schema = {}
    for field_name, field in model_class.model_fields.items():
        # 获取字段的默认值来辅助类型分析
        default_value = field.default if field.default is not ... else None

        # 使用更强大的类型分析函数
        type_info = _analyze_field_type(field.annotation, default_value)

        field_info = {
            "type": type_info["base_type"],
            "title": field.title or field_name,
            "description": field.description or "",
            "default": default_value,
            "required": field.is_required(),
        }

        # 如果是复杂类型，添加额外的类型信息
        if type_info.get("is_complex"):
            field_info["is_complex"] = True
            if type_info.get("element_type"):
                field_info["element_type"] = type_info["element_type"]
            if type_info.get("key_type"):
                field_info["key_type"] = type_info["key_type"]
            if type_info.get("value_type"):
                field_info["value_type"] = type_info["value_type"]
            if type_info.get("field_schema"):
                field_info["field_schema"] = type_info["field_schema"]

        # 添加特殊字段标记
        if hasattr(field, "json_schema_extra") and isinstance(field.json_schema_extra, dict):
            extra_fields = ["is_secret", "is_textarea", "placeholder","is"]
            for extra_field in extra_fields:
                if field.json_schema_extra.get(extra_field):
                    field_info[extra_field] = field.json_schema_extra.get(extra_field)

        schema[field_name] = field_info

    return schema


class UnifiedConfigService:
    """统一配置服务

    提供基于配置键的统一配置管理功能，支持多种类型的配置实例。
    """

    @staticmethod
    def _get_config_instance(config_key: str) -> Optional[ConfigBase]:
        """获取或动态加载配置实例"""
        # 1. 检查缓存
        instance = ConfigManager.get_config(config_key)
        if instance:
            return instance

        # 2. 尝试动态加载
        try:
            from nekro_agent.core.overridable_config import OverridableConfig

            path: Optional[Path] = None

            if config_key.startswith("adapter_override_"):
                adapter_key = config_key.replace("adapter_override_", "")
                path = Path(OsEnv.DATA_DIR) / "configs" / adapter_key / "overrides.yaml"
                instance = OverridableConfig.load_from_path(path)

            elif config_key.startswith("channel_config_"):
                chat_key = config_key.replace("channel_config_", "")
                adapter_key = chat_key.split("-")[0]
                path = CHANNEL_CONFIG_DIR / adapter_key / f"{chat_key}.yaml"
                instance = OverridableConfig.load_from_path(path)

            if instance:
                ConfigManager.register_config(config_key, instance)
                instance.dump_config()  # 确保文件存在
                return instance

        except Exception as e:
            logger.error(f"动态加载配置失败: {config_key}, 错误: {e}")
            return None

        # 3. 降级到静态注册的配置
        return ConfigManager.get_config(config_key)

    @staticmethod
    def get_config_list(config_key: str) -> List[Dict[str, Any]]:
        """获取指定配置的配置列表

        Args:
            config_key: 配置键

        Returns:
            List[Dict[str, Any]]: 配置项列表
        """
        config_obj = UnifiedConfigService._get_config_instance(config_key)
        if not config_obj:
            raise ValueError(f"配置实例不存在: {config_key}")

        return ConfigService.get_config_list(config_obj)

    @staticmethod
    def get_config_item(config_key: str, item_key: str) -> Optional[Dict[str, Any]]:
        """获取指定配置的单个配置项

        Args:
            config_key: 配置键
            item_key: 配置项键名

        Returns:
            Optional[Dict[str, Any]]: 配置项信息，如果不存在则返回 None
        """
        config_obj = UnifiedConfigService._get_config_instance(config_key)
        if not config_obj:
            return None

        return ConfigService.get_config_item(config_obj, item_key)

    @staticmethod
    def set_config_value(config_key: str, item_key: str, value: str) -> Tuple[bool, str]:
        """设置指定配置的配置项值

        Args:
            config_key: 配置键
            item_key: 配置项键名
            value: 配置项值（字符串形式）

        Returns:
            Tuple[bool, str]: (是否成功, 错误消息)
        """
        config_obj = UnifiedConfigService._get_config_instance(config_key)
        if not config_obj:
            return False, f"配置实例不存在: {config_key}"

        return ConfigService.set_config_value(config_obj, item_key, value)

    @staticmethod
    def batch_update_config(config_key: str, configs: Dict[str, str]) -> Tuple[bool, Optional[str]]:
        """批量更新指定配置

        Args:
            config_key: 配置键
            configs: 配置字典，键为配置项名称，值为配置项值

        Returns:
            Tuple[bool, Optional[str]]: (是否成功, 错误消息)
        """
        config_obj = UnifiedConfigService._get_config_instance(config_key)
        if not config_obj:
            return False, f"配置实例不存在: {config_key}"

        return ConfigService.batch_update_config(config_obj, configs)

    @staticmethod
    def save_config(config_key: str, file_path: Optional[Path] = None) -> Tuple[bool, Optional[str]]:
        """保存指定配置到文件

        Args:
            config_key: 配置键
            file_path: 配置文件路径（可选，使用配置实例的默认路径）

        Returns:
            Tuple[bool, Optional[str]]: (是否成功, 错误消息)
        """
        config_obj = UnifiedConfigService._get_config_instance(config_key)
        if not config_obj:
            return False, f"配置实例不存在: {config_key}"

        return ConfigService.save_config(config_obj, file_path)

    @staticmethod
    def reload_config(config_key: str, file_path: Optional[Path] = None) -> Tuple[bool, Optional[str]]:
        """重新加载指定配置

        Args:
            config_key: 配置键
            file_path: 配置文件路径（可选）

        Returns:
            Tuple[bool, Optional[str]]: (是否成功, 错误消息)
        """
        try:
            config_obj = UnifiedConfigService._get_config_instance(config_key)
            if not config_obj:
                return False, f"配置实例不存在: {config_key}"

            # 获取要重载的路径
            reload_path = file_path or config_obj.get_config_file_path()
            if not reload_path:
                return False, f"重载失败: 配置文件路径未知: {config_key}"

            # 获取配置类并从路径加载
            config_class = config_obj.__class__
            new_config = config_class.load_from_path(reload_path)

            # 注册新实例并加载到env
            ConfigManager.register_config(config_key, new_config)
            new_config.load_config_to_env()

        except Exception as e:
            logger.error(f"重新加载配置失败: {config_key}, 错误: {e}")
            return False, str(e)
        else:
            return True, None

    @staticmethod
    def get_all_config_keys() -> List[str]:
        """获取所有已注册的配置键"""
        return ConfigManager.get_all_config_keys()

    @staticmethod
    def get_config_info(config_key: str) -> Optional[Dict[str, Any]]:
        """获取配置基本信息

        Args:
            config_key: 配置键

        Returns:
            Optional[Dict[str, Any]]: 配置信息
        """
        config_obj = UnifiedConfigService._get_config_instance(config_key)
        if not config_obj:
            return None

        return {
            "config_key": config_key,
            "config_class": config_obj.__class__.__name__,
            "config_file_path": (str(config_obj.get_config_file_path()) if config_obj.get_config_file_path() else None),
            "config_type": _get_config_type(config_key),
            "field_count": len(config_obj.__class__.model_fields),
        }


class ConfigService:
    """配置服务

    提供统一的配置管理功能，支持系统配置和插件配置。
    """

    @staticmethod
    def get_config_list(config_obj: ConfigBase) -> List[Dict[str, Any]]:
        """获取配置列表

        Args:
            config_obj: 配置对象实例

        Returns:
            List[Dict[str, Any]]: 配置项列表
        """
        result = []

        for key, field in config_obj.__class__.model_fields.items():
            # 获取字段值
            value = getattr(config_obj, key, None)

            # 分析字段类型
            type_info = _analyze_field_type(field.annotation, value)

            # 构建配置项
            config_item = {
                "key": key,
                "value": value,
                "type": type_info["base_type"],
                "is_complex": type_info.get("is_complex", False),
                "element_type": type_info.get("element_type"),
                "key_type": type_info.get("key_type"),
                "value_type": type_info.get("value_type"),
                "field_schema": type_info.get("field_schema"),
                "title": config_obj.__class__.get_field_title(key) or key,
                "description": field.description,
                "placeholder": config_obj.__class__.get_field_placeholder(key),
            }

            # 添加特殊字段标记
            if hasattr(field, "json_schema_extra") and isinstance(field.json_schema_extra, dict):
                extra_fields = [
                    "is_secret",
                    "is_textarea",
                    "required",
                    "is_hidden",
                    "ref_model_groups",
                    "model_type",
                    "sub_item_name",
                    "enable_toggle",
                    "overridable",
                    "load_to_sysenv",
                    "load_sysenv_as",
                    "load_to_nonebot_env",
                    "load_nbenv_as",
                    "is_need_restart",
                ]
                for extra_field in extra_fields:
                    if field.json_schema_extra.get(extra_field):
                        config_item[extra_field] = field.json_schema_extra.get(extra_field)

            # 添加枚举选项
            enum_values = _get_field_enum(field.annotation)
            if enum_values:
                config_item["enum"] = enum_values

            result.append(config_item)

        return result

    @staticmethod
    def get_config_item(config_obj: ConfigBase, key: str) -> Optional[Dict[str, Any]]:
        """获取单个配置项

        Args:
            config_obj: 配置对象实例
            key: 配置项键名

        Returns:
            Optional[Dict[str, Any]]: 配置项信息，如果不存在则返回 None
        """
        if key not in config_obj.model_dump():
            return None

        field = config_obj.__class__.model_fields.get(key)
        if not field:
            return None

        # 获取字段值
        value = getattr(config_obj, key)

        # 分析字段类型
        type_info = _analyze_field_type(field.annotation, value)

        # 构建配置项
        config_item = {
            "key": key,
            "value": value,
            "type": type_info["base_type"],
            "is_complex": type_info.get("is_complex", False),
            "element_type": type_info.get("element_type"),
            "title": config_obj.__class__.get_field_title(key) or key,
            "description": field.description,
            "placeholder": config_obj.__class__.get_field_placeholder(key),
        }

        # 添加特殊字段标记
        if hasattr(field, "json_schema_extra") and isinstance(field.json_schema_extra, dict):
            extra_fields = [
                "is_secret",
                "is_textarea",
                "required",
                "is_hidden",
                "ref_model_groups",
                "model_type",
                "sub_item_name",
                "enable_toggle",
                "overridable",
                "load_to_sysenv",
                "load_sysenv_as",
                "load_to_nonebot_env",
                "load_nbenv_as",
                "is_need_restart",
            ]
            for extra_field in extra_fields:
                if field.json_schema_extra.get(extra_field):
                    config_item[extra_field] = field.json_schema_extra.get(extra_field)

        # 添加枚举选项
        enum_values = _get_field_enum(field.annotation)
        if enum_values:
            config_item["enum"] = enum_values

        return config_item

    @staticmethod
    def set_config_value(config_obj: ConfigBase, key: str, value: str) -> Tuple[bool, str]:
        """设置配置项值

        Args:
            config_obj: 配置对象实例
            key: 配置项键名
            value: 配置项值（字符串形式）

        Returns:
            Tuple[bool, str]: (是否成功, 错误消息)
        """
        if key not in config_obj.model_dump():
            return False, f"配置项 {key} 不存在"

        try:
            current_value = getattr(config_obj, key)

            # 根据类型转换值
            if isinstance(current_value, bool):
                if value.lower() in ["true", "1", "yes", "t", "y"]:
                    setattr(config_obj, key, True)
                elif value.lower() in ["false", "0", "no", "f", "n"]:
                    setattr(config_obj, key, False)
                else:
                    return False, "布尔值只能是 true 或 false"
            elif isinstance(current_value, (int, float)):
                setattr(config_obj, key, type(current_value)(value))
            elif isinstance(current_value, str):
                setattr(config_obj, key, value)
            elif isinstance(current_value, (list, dict)):
                try:
                    parsed_value = json.loads(value)
                    # 简单类型检查
                    if isinstance(current_value, list) and not isinstance(parsed_value, list):
                        return False, "输入必须是有效的列表格式"
                    if isinstance(current_value, dict) and not isinstance(parsed_value, dict):
                        return False, "输入必须是有效的对象格式"

                    # 对于复杂类型（如 List[Pydantic模型]），尝试使用 Pydantic 验证
                    field_info = config_obj.__class__.model_fields.get(key)
                    if field_info:
                        test_data = {key: parsed_value}
                        config_obj.__class__.model_validate(test_data)
                        setattr(config_obj, key, parsed_value)
                    else:
                        setattr(config_obj, key, parsed_value)

                except json.JSONDecodeError:
                    return False, "输入必须是有效的 JSON 格式"
                except (ValueError, TypeError) as e:
                    return False, f"数据格式转换失败: {e}"
            else:
                return False, f"不支持的配置类型: {type(current_value)}"

        except ValueError as e:
            return False, f"配置值类型错误: {e}"
        except Exception as e:
            return False, f"设置配置值时发生错误: {e}"
        else:
            return True, ""

    @staticmethod
    def batch_update_config(config_obj: ConfigBase, configs: Dict[str, str]) -> Tuple[bool, Optional[str]]:
        """批量更新配置

        Args:
            config_obj: 配置对象实例
            configs: 配置字典，键为配置项名称，值为配置项值

        Returns:
            Tuple[bool, Optional[str]]: (是否成功, 错误消息)
        """
        for key, value in configs.items():
            if key not in config_obj.model_dump():
                return False, f"配置项不存在: {key}"

            success, message = ConfigService.set_config_value(config_obj, key, value)
            if not success:
                return False, message

        return True, None

    @staticmethod
    def save_config(config_obj: ConfigBase, file_path: Optional[Path] = None) -> Tuple[bool, Optional[str]]:
        """保存配置到文件

        Args:
            config_obj: 配置对象实例
            file_path: 配置文件路径

        Returns:
            Tuple[bool, Optional[str]]: (是否成功, 错误消息)
        """
        try:
            config_obj.dump_config(file_path)
        except Exception as e:
            logger.error(f"保存配置失败: {e}")
            return False, str(e)
        else:
            return True, None


def _get_field_type(value: Any) -> str:
    """根据值获取字段类型

    Args:
        value: 字段值

    Returns:
        str: 字段类型名称
    """
    if isinstance(value, bool):
        return "bool"
    if isinstance(value, int):
        return "int"
    if isinstance(value, float):
        return "float"
    if isinstance(value, list):
        return "list"
    if isinstance(value, dict):
        return "dict"
    return "str"


def _get_field_enum(field_type: Any) -> Optional[List[str]]:
    """获取字段的枚举选项

    Args:
        field_type: 字段类型

    Returns:
        Optional[List[str]]: 枚举选项列表，如果不是枚举类型则返回 None
    """
    try:
        if hasattr(field_type, "__origin__") and field_type.__origin__ is Literal:
            return list(get_args(field_type))
    except Exception:
        pass
    return None


def _get_config_type(config_key: str) -> str:
    """根据配置键获取配置类型

    Args:
        config_key: 配置键

    Returns:
        str: 配置类型
    """
    if config_key == "system":
        return ConfigType.SYSTEM.value
    if config_key.startswith("plugin."):
        return ConfigType.PLUGIN.value
    if config_key == "model_groups":
        return ConfigType.MODEL_GROUPS.value
    if config_key.startswith("adapter."):
        return ConfigType.ADAPTER.value
    if config_key.startswith("session."):
        return ConfigType.SESSION.value
    return "unknown"



import asyncio
from typing import TYPE_CHECKING

from nekro_agent.adapters.interface.collector import collect_message
from nekro_agent.adapters.interface.schemas.extra import PlatformMessageExt
from nekro_agent.adapters.interface.schemas.platform import (
    PlatformChannel,
    PlatformMessage,
    PlatformUser,
)
from nekro_agent.core.logger import logger
from nekro_agent.schemas.chat_message import (
    ChatMessageSegment,
    ChatMessageSegmentFile,
    ChatMessageSegmentImage,
    ChatMessageSegmentType,
    ChatType,
)

if TYPE_CHECKING:
    from .adapter import DiscordAdapter


try:
    import discord
except ImportError as e:
    raise ImportError("The 'discord' package is not installed. Please install it with: poetry install -E discord") from e


class DiscordClient:
    """
    一个用于管理与 Discord Gateway 连接的客户端。
    """

    def __init__(self, token: str, adapter: "DiscordAdapter"):
        self._token = token
        self._adapter = adapter
        self._intents = discord.Intents.default()
        self._intents.message_content = True  # 需要开启消息内容权限
        self._client = discord.Client(intents=self._intents)

        self._setup_events()

    @property
    def user(self) -> discord.ClientUser | None:
        return self._client.user

    def get_channel(self, channel_id: int) -> discord.abc.GuildChannel | discord.Thread | discord.abc.PrivateChannel | None:
        return self._client.get_channel(channel_id)

    def get_user(self, user_id: int) -> discord.User | None:
        return self._client.get_user(user_id)

    async def fetch_user(self, user_id: int) -> discord.User:
        return await self._client.fetch_user(user_id)

    async def fetch_channel(self, channel_id: int) -> discord.abc.GuildChannel | discord.Thread | discord.abc.PrivateChannel:
        return await self._client.fetch_channel(channel_id)

    def _setup_events(self):
        @self._client.event
        async def on_ready():
            logger.success(f"Discord Bot is ready. Logged in as {self._client.user}")

        @self._client.event
        async def on_message(message: discord.Message):
            # 忽略来自 bot 自身的消息
            if message.author == self._client.user:
                return

            chat_key = self._adapter.build_chat_key(str(message.channel.id))
            segments: list[ChatMessageSegment] = []
            content_text = message.content

            # 1. 处理附件
            for attachment in message.attachments:
                file_name = attachment.filename
                if attachment.content_type and "image" in attachment.content_type:
                    segment = await ChatMessageSegmentImage.create_from_url(
                        url=attachment.url,
                        from_chat_key=chat_key,
                        file_name=file_name,
                    )
                else:
                    segment = await ChatMessageSegmentFile.create_from_url(
                        url=attachment.url,
                        from_chat_key=chat_key,
                        file_name=file_name,
                    )
                segments.append(segment)
                # 将文件信息附加到主文本内容后，确保 Agent 能感知到
                if segment.text not in content_text:
                    content_text += f" {segment.text}"

            # 2. 处理文本内容
            if message.content:
                segments.append(ChatMessageSegment(type=ChatMessageSegmentType.TEXT, text=message.content))

            if not segments:
                logger.debug("空消息，已忽略")
                return

            # 3. 构建 PlatformUser
            user = PlatformUser(
                platform_name="discord",
                user_id=str(message.author.id),
                user_name=message.author.display_name,
                user_avatar=str(message.author.display_avatar.url),
            )

            # 4. 构建 PlatformChannel
            channel_type = ChatType.PRIVATE if isinstance(message.channel, discord.DMChannel) else ChatType.GROUP
            channel = PlatformChannel(
                channel_id=str(message.channel.id),
                channel_name=str(message.channel),  # For DMs, this will be the user's name
                channel_type=channel_type,
            )

            # 5. 构建 PlatformMessage
            ref_msg_id = str(message.reference.message_id) if message.reference else ""
            platform_message = PlatformMessage(
                message_id=str(message.id),
                sender_id=str(message.author.id),
                sender_name=message.author.display_name,
                content_text=content_text.strip(),
                content_data=segments,
                is_tome=self._client.user in message.mentions or isinstance(message.channel, discord.DMChannel),
                sender_nickname=message.author.display_name,
                ext_data=PlatformMessageExt(ref_msg_id=ref_msg_id),
            )

            # 6. 分发消息
            await collect_message(
                adapter=self._adapter,
                platform_channel=channel,
                platform_user=user,
                platform_message=platform_message,
            )

    async def start(self):
        """启动客户端并连接到 Discord"""
        logger.info("Starting Discord client...")
        # discord.py 的 login 和 connect 方法是阻塞的，
        # 需要使用 start 方法在后台任务中运行它。
        # start 会自动处理 login 和 connect。
        asyncio.create_task(self._client.start(self._token))

    async def stop(self):
        """关闭客户端连接"""
        logger.info("Stopping Discord client...")
        await self._client.close()



from nekro_agent.core.config import CoreConfig
from nekro_agent.core.config import config as system_config
from nekro_agent.core.overridable_config import OverridableConfig
from nekro_agent.services.config_service import UnifiedConfigService


class ConfigResolver:
    """配置解析器
    根据 会话 > 适配器 > 系统 的优先级解析最终生效的配置。
    """

    async def get_effective_config(self, chat_key: str) -> CoreConfig:
        """获取指定会话的最终有效配置

        Args:
            chat_key: 会话标识

        Returns:
            CoreConfig: 一个包含已解析配置的新 CoreConfig 实例
        """
        # 1. 获取所有配置层
        adapter_key = chat_key.split("-")[0]

        # 系统配置是基础
        effective_config_data = system_config.model_dump()

        # 使用 UnifiedConfigService 加载适配器和会话的覆盖配置
        # 这会利用缓存或从文件动态加载
        adapter_overrides = UnifiedConfigService._get_config_instance(f"adapter_override_{adapter_key}")  # noqa: SLF001
        channel_overrides = UnifiedConfigService._get_config_instance(f"channel_config_{chat_key}")  # noqa: SLF001

        # 2. 逐字段应用覆盖逻辑
        for field_name in effective_config_data:
            # 检查会话级覆盖
            if isinstance(channel_overrides, OverridableConfig):
                enable_key = f"enable_{field_name}"
                if hasattr(channel_overrides, enable_key) and getattr(channel_overrides, enable_key):
                    effective_config_data[field_name] = getattr(channel_overrides, field_name)
                    continue  # 已被会话覆盖，继续下一个字段

            # 检查适配器级覆盖
            if isinstance(adapter_overrides, OverridableConfig):
                enable_key = f"enable_{field_name}"
                if hasattr(adapter_overrides, enable_key) and getattr(adapter_overrides, enable_key):
                    effective_config_data[field_name] = getattr(adapter_overrides, field_name)
                    continue  # 已被适配器覆盖，继续下一个字段

        # 使用最终解析出的数据创建一个新的 CoreConfig 实例
        return CoreConfig(**effective_config_data)


# 创建配置解析器单例
config_resolver = ConfigResolver()



import importlib
import logging

import weave
from nonebot import get_app, get_driver
from nonebot.adapters.onebot.v11 import Bot
from nonebot.plugin import PluginMetadata
from pydantic import BaseModel

from nekro_agent.adapters import cleanup_adapters, init_adapters
from nekro_agent.core.args import Args
from nekro_agent.core.config import config
from nekro_agent.core.database import init_db
from nekro_agent.core.logger import logger
from nekro_agent.routers import mount_api_routes, mount_middlewares
from nekro_agent.services.festival_service import festival_service
from nekro_agent.services.mail.mail_service import send_bot_status_email
from nekro_agent.services.plugin.collector import init_plugins
from nekro_agent.services.timer_service import timer_service
from nekro_agent.systems.cloud.scheduler import start_telemetry_task

logging.getLogger("passlib").setLevel(logging.ERROR)


class _Config(BaseModel):
    pass


if config.WEAVE_ENABLED:
    logger.info("正在连接 Weave 服务...")
    try:
        weave.init(config.WEAVE_PROJECT_NAME)
        logger.success("Weave 服务连接成功")
    except Exception as e:
        logger.error(f"Weave 服务连接失败: {e}")

# 在应用启动前挂载中间件和主路由
# 这是确保应用生命周期正确的唯一方法
app = get_app()
mount_middlewares(app)
mount_api_routes(app)


@get_driver().on_startup
async def on_startup():
    # 启动时不再挂载主路由，它们已在启动前挂载完毕
    app = get_app()

    # 初始化数据库、适配器和插件
    await init_db()
    await init_adapters(app)
    await init_plugins()

    # 🎯 关键修复：在静态文件挂载之前挂载插件路由！
    # 初始化插件路由管理器并挂载插件路由
    try:
        from nekro_agent.services.plugin.collector import plugin_collector
        from nekro_agent.services.plugin.router_manager import plugin_router_manager

        # 绑定FastAPI应用实例到路由管理器
        plugin_router_manager.set_app(app)

        # 挂载所有启用的插件路由
        plugins_with_router = plugin_collector.get_plugins_with_router()
        success_count = 0
        for plugin in plugins_with_router:
            if plugin_router_manager.mount_plugin_router(plugin):
                success_count += 1

        logger.info(f"插件路由热挂载完成，成功挂载 {success_count} 个插件的路由")

    except Exception as e:
        logger.exception(f"初始化插件路由管理器失败: {e}")

    await timer_service.start()
    logger.info("Timer service initialized")

    # 初始化节日提醒
    await festival_service.init_festivals()
    logger.info("Festival service initialized")

    # 遥测任务
    start_telemetry_task()


@get_driver().on_shutdown
async def on_shutdown():
    await timer_service.stop()
    await cleanup_adapters(get_app())

    try:
        from nekro_agent.services.plugin.collector import plugin_collector

        await plugin_collector.cleanup_all_plugins()
    except Exception as e:
        logger.exception(f"清理插件时发生错误: {e}")

    logger.info("Timer service stopped")


@get_driver().on_bot_connect
async def on_bot_connect(bot: Bot):
    adapter: str = bot.adapter.get_name()
    bot_id: str = bot.self_id
    await send_bot_status_email(adapter, bot_id, True)


@get_driver().on_bot_disconnect
async def on_bot_disconnect(bot: Bot):
    adapter: str = bot.adapter.get_name()
    bot_id: str = bot.self_id
    await send_bot_status_email(adapter, bot_id, False)


__plugin_meta__ = PluginMetadata(
    name="nekro-agent",
    description="集代码执行/高度可扩展性为一体的聊天机器人，应用了容器化技术快速构建沙盒环境",
    usage="",
    type="application",
    homepage="https://github.com/KroMiose/nekro-agent",
    supported_adapters={"~onebot.v11"},
    config=_Config,
)

global_config = get_driver().config  # 我觉得这玩意可以删掉 没人用


# 启动 Api 服务进程
# threading.Thread(target=start, daemon=True).start()

if Args.LOAD_TEST:
    logger.success("Plugin load tested successfully")
    exit(0)



import asyncio

from fastapi import APIRouter, Depends

from nekro_agent.core.logger import logger
from nekro_agent.core.os_env import OsEnv
from nekro_agent.models.db_user import DBUser
from nekro_agent.schemas.message import Ret
from nekro_agent.services.user.deps import get_current_active_user
from nekro_agent.services.user.perm import Role, require_role
from nekro_agent.tools.docker_util import restart_self

router = APIRouter(prefix="/restart", tags=["Restart"])


@router.post("", summary="重启系统")
@require_role(Role.Admin)
async def restart_system(
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """重启整个系统服务"""

    try:
        # 执行重启操作
        if OsEnv.RUN_IN_DOCKER:
            logger.info("在Docker环境中执行重启操作")
            result = await restart_self()
            if result:
                return Ret.success(msg="重启命令已发送")
            return Ret.error(msg="重启命令发送失败")
        logger.info("在非Docker环境中执行重启操作")
        return Ret.error(msg="非Docker环境下无法自动重启，请手动重启应用")
    except Exception as e:
        logger.error(f"重启系统失败: {e}")
        return Ret.error(msg=f"重启失败: {e!s}")
